import sys, time, serial, subprocess, shlex, traceback, glob
from os import path

nucleo_stlink_serials = {}
nucleo_debug_sessions = {}

def list_devices(host_os):
    return glob.glob('/dev/stm32*-*')

def exist(device):
    return path.exists(device)

def new_device(device):
    info = subprocess.check_output(['udevadm', 'info', '-q', 'property', '-n', device])
    vendor_id = None
    model_id  = None
    serial_id = None
    for line in info.split('\n'):
        line.replace('\n', '')
        if line.startswith('ID_VENDOR_ID='):
            vendor_id = line.replace('ID_VENDOR_ID=', '')
        if line.startswith('ID_MODEL_ID='):
            model_id = line.replace('ID_MODEL_ID=', '')
        if line.startswith('ID_SERIAL_SHORT='):
            serial_id = line.replace('ID_SERIAL_SHORT=', '')
    if vendor_id != '0483' or model_id != '374b' or serial_id == None:
        print('stm32_nucleo: parse stlink serial_id for {0} failed'.format(device))
        print('stm32_nucleo: vendor_id:{0} model_id:{1} serial_id:{2}'.format(repr(vendor_id), repr(model_id), repr(serial_id)))
        return None
    if len(serial_id) > 15:
        serial_id = serial_id[:15]
    serial_hexid = ''
    for c in serial_id:
        serial_hexid += '{0:02x}'.format(ord(c))
    nucleo_stlink_serials[device] = serial_hexid
    try:
        ser = serial.Serial(device, 115200, timeout = 0.02)
        subprocess.call(['st-flash', '--serial', nucleo_stlink_serials[device], 'reset'])
    except:
        ser = None
        print('stm32_nucleo: open {0} error'.format(device))
    return ser

def erase(device):
    retry = 3
    error = 'fail'
    if device not in nucleo_stlink_serials:
        return error
    while retry > 0:
        script = ['st-flash', '--serial', nucleo_stlink_serials[device], 'erase']
        ret = subprocess.call(script)
        if ret == 0:
            error =  'success'
            break
        retry -= 1
        time.sleep(4)
    return error

def program(device, address, file):
    retry = 3
    error = 'fail'
    if device not in nucleo_stlink_serials:
        return error
    while retry > 0:
        script = ['st-flash', '--serial', nucleo_stlink_serials[device]]
        script += ['write', file, address]
        ret = subprocess.call(script)
        if ret == 0:
            error =  'success'
            break
        retry -= 1
        time.sleep(4)
    return error

def control(device, operation):
    ret = 'fail'
    if device not in nucleo_stlink_serials:
        return ret
    try:
        if operation == 'reset':
            subprocess.call(['st-flash', '--serial', nucleo_stlink_serials[device], 'reset'])
            ret = 'success'
        elif operation == 'stop':
            ret = 'fail'
        elif operation == 'start':
            ret = 'fail'
    except:
        pass
    return ret

def debug_start(device, port):
    if device not in nucleo_stlink_serials:
        return 'nonexist'
    if device in nucleo_debug_sessions:
        return 'busy'

    for device in nucleo_debug_sessions:
        if nucleo_debug_sessions[device]['port'] == port:
            return 'port_in_use'
    logfile = path.join(path.expanduser('~'), '.udclient', path.basename(device) + '-debuglog.txt')
    try:
        flog = open(logfile, 'a+')
    except:
        traceback.print_exc()
        return 'open_log_fail'
    command = 'st-util -m --serial {0} -p {1}'.format(nucleo_stlink_serials[device], port)
    command = shlex.split(command)
    p = subprocess.Popen(command, stdout=flog, stderr=flog)
    time.sleep(0.2)
    if p.poll() != None:
        return 'fail'
    nucleo_debug_sessions[device] = {'process': p, 'port': port, 'flog':flog }
    return 'success'

def debug_stop(device):
    if device not in nucleo_debug_sessions:
        return 'fail'
    nucleo_debug_sessions[device]['process'].kill()
    nucleo_debug_sessions[device]['flog'].close()
    nucleo_debug_sessions.pop(device)
    return 'success'




import sys
import click
from aos.util import get_host_os, ProcessException
from serial.tools import miniterm

# Make command
@click.command("monitor", short_help="Serial port monitor")
@click.argument("port", required=True, nargs=1)
@click.argument("baud", required=True, nargs=1)
def cli(port, baud):
    """ Open a simple serial monitor. """
    args = ['miniterm', port, baud]
    host_os = get_host_os()
    if host_os == 'Win32':
        args += ['--eol', 'CRLF']
    elif host_os == 'OSX':
        args += ['--eol', 'CR']
    else:
        args += ['--eol', 'LF']

    sys.argv = args
    try:
        miniterm.main()
    except ProcessException as e:
        raise e



import click
from aos.util import popen

# Upgrade command
@click.command(
    "upgrade", short_help="Upgrade aos-cube to latest")
def cli():
    """ Run pip upgrade process to keep aos-cube up-to-date. """
    cmd = ["pip", "install", "--upgrade", "aos-cube"]
    try:
        ret = popen(cmd)
        if ret != 0:
            cmd.insert(3, "--no-cache-dir")
            popen(cmd)
    except Exception as e:
        raise e



import os

# Default paths to Mercurial and Git
hg_cmd = 'hg'
git_cmd = 'git'

ignores = [
    "make",
    "make.exe",
    "Makefile",
    "build",
    ".cproject",
    ".gdbinit",
    ".openocd_cfg",
    ".project",
    "aos",
    ".aos",
]

toolchains = {
    'arm-none-eabi':{
        'name': 'gcc-arm-none-eabi',
        'path': 'build/compiler/gcc-arm-none-eabi',
        'command': 'arm-none-eabi-gcc',
        'version': 'all',
        'use_global': True,
        'Win32_url':'https://gitee.com/alios-things/gcc-arm-none-eabi-win32.git',
        'Linux32_url': 'https://gitee.com/alios-things/gcc-arm-none-eabi-linux.git',
        'Linux64_url': 'https://gitee.com/alios-things/gcc-arm-none-eabi-linux.git',
        'OSX_url': 'https://gitee.com/alios-things/gcc-arm-none-eabi-osx.git',
        },
    'xtensa-esp32':{
        'name': 'gcc-xtensa-esp32',
        'path': 'build/compiler/gcc-xtensa-esp32',
        'command': 'xtensa-esp32-elf-gcc',
        'version': '5.2.0',
        'use_global': True,
        'Win32_url': 'https://gitee.com/alios-things/gcc-xtensa-esp32-win32.git',
        'Linux32_url': 'https://gitee.com/alios-things/gcc-xtensa-esp32-linux.git',
        'Linux64_url': 'https://gitee.com/alios-things/gcc-xtensa-esp32-linux.git',
        'OSX_url': 'https://gitee.com/alios-things/gcc-xtensa-esp32-osx.git',
        },
    'xtensa-lx106':{
        'name': 'gcc-xtensa-lx106',
        'path': 'build/compiler/gcc-xtensa-lx106',
        'command': 'xtensa-lx106-elf-gcc',
        'version': '4.8.2',
        'use_global': True,
        'Win32_url': 'https://gitee.com/alios-things/gcc-xtensa-lx106-win32.git',
        'Linux32_url': 'https://gitee.com/alios-things/gcc-xtensa-lx106-linux.git',
        'Linux64_url': 'https://gitee.com/alios-things/gcc-xtensa-lx106-linux.git',
        'OSX_url': 'https://gitee.com/alios-things/gcc-xtensa-lx106-osx.git',
        },
    'csky-abiv2': {
        'name': 'gcc-csky-abiv2',
        'path': 'build/compiler/gcc-csky-abiv2',
        'command': 'csky-abiv2-elf-gcc',
        'version': 'all',
        'use_global': True,
        'Win32_url': 'https://gitee.com/alios-things/gcc-csky-abiv2-win32.git',
        'Linux32_url': 'https://gitee.com/alios-things/gcc-csky-abiv2-linux.git',
        'Linux64_url': 'https://gitee.com/alios-things/gcc-csky-abiv2-linux.git',
        'OSX_url': '',
        },

    'arm-rockchip-linux-gnueabihf': {
        'name': 'gcc-arm-rockchip-linux-gnueabihf',
        'path': 'build/compiler/usr',
        'path_specific': True,
        'command': 'arm-rockchip-linux-gnueabihf-gcc',
        'version': 'all',
        'use_global': True,
        'Win32_url': '',
        'Linux32_url': 'https://gitee.com/alios-things/arm-rockchip-linux-gnueabihf-linux.git',
        'Linux64_url': 'https://gitee.com/alios-things/arm-rockchip-linux-gnueabihf-linux.git',
        'OSX_url': '',
        },

    'nds32le-elf-newlib-v3': {
        'name': 'nds32le-elf-newlib-v3',
        'path': 'build/compiler/nds32le-elf-newlib-v3',
        'path_specific': True,
        'command': 'nds32le-elf-gcc',
        'version': 'all',
        'use_global': True,
        'Win32_url': '',
        'Linux32_url': 'https://gitee.com/alios-things/gcc-nds32le-linux.git',
        'Linux64_url': 'https://gitee.com/alios-things/gcc-nds32le-linux.git',
        'OSX_url': '',
        },

    'openocd': {
        'name': 'OpenOCD',
        'path': 'build/OpenOCD',
        'command': 'openocd',
        'version': '0.10.0',
        'use_global': False,
        'Win32_url': 'https://gitee.com/alios-things/openocd-win32.git',
        'Linux32_url': '',
        'Linux64_url': 'https://gitee.com/alios-things/openocd-linux64.git',
        'OSX_url': 'https://gitee.com/alios-things/openocd-osx.git',
    }
}

boards = {
'amebaz_dev':[toolchains['arm-none-eabi']],
'atsame54-xpro':[toolchains['arm-none-eabi']],
'b_l475e':[toolchains['arm-none-eabi']],
'bk7231devkitc':[toolchains['arm-none-eabi']],
'bk7231udevkitc':[toolchains['arm-none-eabi']],
'developerkit':[toolchains['arm-none-eabi']],
'eml3047':[toolchains['arm-none-eabi']],
'esp32devkitc':[toolchains['xtensa-esp32']],
'esp8266':[toolchains['xtensa-lx106']],
'frdmkl27z':[toolchains['arm-none-eabi']],
'hobbit1_evb':[toolchains['csky-abiv2']],
'dh5021a_evb':[toolchains['csky-abiv2']],
'cb2201':[toolchains['csky-abiv2']],
'lpcxpresso54102':[toolchains['arm-none-eabi']],
'mk1101':[toolchains['arm-none-eabi']],
'mk3060':[toolchains['arm-none-eabi']],
'mk3080':[toolchains['arm-none-eabi']],
'mk3165':[toolchains['arm-none-eabi']],
'mk3166':[toolchains['arm-none-eabi']],
'mk3239':[toolchains['arm-none-eabi']],
'pca10056':[toolchains['arm-none-eabi']],
'pca10040':[toolchains['arm-none-eabi']],
'starterkit':[toolchains['arm-none-eabi']],
'stm32f769i-discovery':[toolchains['arm-none-eabi']],
'stm32f412zg-nucleo':[toolchains['arm-none-eabi']],
'stm32l073rz-nucleo':[toolchains['arm-none-eabi']],
'stm32l432kc-nucleo':[toolchains['arm-none-eabi']],
'stm32l433rc-nucleo':[toolchains['arm-none-eabi']],
'stm32l476rg-nucleo':[toolchains['arm-none-eabi']],
'stm32l496g-discovery':[toolchains['arm-none-eabi']],
'sv6266_evb':[toolchains['nds32le-elf-newlib-v3']],
'msp432p4111launchpad':[toolchains['arm-none-eabi']],
'xr871evb':[toolchains['arm-none-eabi']],
'rk1108':[toolchains['arm-rockchip-linux-gnueabihf']],
'uno-91h':[toolchains['arm-none-eabi']],
}

# reference to local (unpublished) repo - dir#rev
regex_local_ref = r'^([\w.+-][\w./+-]*?)/?(?:#(.*))?$'
# reference to repo - url#rev
regex_url_ref = r'^(.*/([\w.+-]+)(?:\.\w+)?)/?(?:#(.*?)?)?$'

# git url (no #rev)
regex_git_url = r'^(git\://|ssh\://|https?\://|)(([^/:@]+)(\:([^/:@]+))?@)?([^/:]+)[:/](.+?)(\.git|\/?)$'
# hg url (no #rev)
regex_hg_url = r'^(file|ssh|https?)://([^/:]+)/([^/]+)/?([^/]+?)?$'

# aos url is subset of hg. aos doesn't support ssh transport
regex_aos_url = r'^(https?)://([\w\-\.]*aos\.(co\.uk|org|com))/(users|teams)/([\w\-]{1,32})/(repos|code)/([\w\-]+)/?$'
# aos sdk builds url
regex_build_url = r'^(https?://([\w\-\.]*aos\.(co\.uk|org|com))/(users|teams)/([\w\-]{1,32})/(repos|code)/([\w\-]+))/builds/?([\w\-]{6,40}|tip)?/?$'

# default aos url
aos_os_url = 'https://github.com/alibaba/AliOS-Things.git'
# default aos component url
aos_lib_url = 'https://aos.org/users/aos_official/code/aos/builds/'
# aos SDK tools needed for programs based on aos SDK component
aos_sdk_tools_url = 'https://aos.org/users/aos_official/code/aos-sdk-tools'
# open_ocd_zip
open_ocd_url = 'https://files.alicdn.com/tpsservice/27ba2d597a43abfca94de351dae65dff.zip'

# verbose logging
verbose = False
very_verbose = False
install_requirements = True
cache_repositories = True

# stores current working directory for recursive operations
cwd_root = ""

eclispe_project_dir = 'aos/makefiles/eclipse_project'

APP_PATH = 'app_path'
PROGRAM_PATH = 'program_path'
AOS_SDK_PATH = 'AOS_SDK_PATH'
OS_PATH = 'os_path'
OS_NAME = 'AliOS-Things'
PATH_TYPE = 'path_type'
AOS_COMPONENT_BASE_URL = 'https://github.com/AliOS-Things'
CUBE_MAKEFILE = 'cube.mk'
CUBE_MODIFY = 'cube_modify'
REMOTE_PATH = 'remote'
OS_CONFIG = "project.ini"
OS_REPO = "http://116.62.245.240/AliOSThings-2-packages/"
OS_DEF_COMPS = [ "buildsystem", "system_include"]

# aos ota config
OTA_SERVER = "116.62.245.240"
OTA_EMQ_PORT = 17173
OTA_EMQ_TOKEN = "QWxpT1MtVGhpbmdzLXVkZXZ8dWRldiFAIyQl"
OTA_WEBSERVER_PORT = 7001
OTA_UDEBUG_LIB = 'udev.a'



#!/usr/bin/env python

# Copyright (c) 2016 aos Limited, All Rights Reserved
# SPDX-License-Identifier: Apache-2.0

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied.

import os, sys
import json
import click

from aos.util import *
from aos.repo import *
from aos.managers.comp import CompManager
import ConfigParser

@click.group(short_help="Component Manager")
@click.pass_context
def cli(ctx):
    ctx.obj = CompManager()

#
# Functions
#
def print_component_status(json_format):
    cwd_type = Repo().pathtype()
    os_name = OS_NAME

    if cwd_type == 'program':
        pd = Program(os.getcwd())
        aos_path = pd.get_cfg(OS_PATH)
        aos_remote_components = get_aos_components(pd.get_cfg(REMOTE_PATH))
        os_name = 'aos'
    else:
        if os.path.isdir("kernel/rhino"):
            aos_path = os.getcwd()
        else:
            aos_path = Global().get_cfg(AOS_SDK_PATH)

    aos_local_components = get_aos_components(aos_path)
    if json_format:
        json_components_dict = {}

        if cwd_type == 'program':
            pd = Program(os.getcwd())
            with cd(pd.get_cfg(PROGRAM_PATH)):
                cube_add_components = None
                cube_remove_components = None
                with open(CUBE_MAKEFILE, 'r') as fin:
                    for line in fin:
                        if line.startswith('CUBE_ADD_COMPONENTS :='):
                            cube_add_components = re.split('\s+', line[23:].strip())

                        if line.startswith('CUBE_REMOVE_COMPONENTS :='):
                            cube_remove_components = re.split('\s+', line[26:].strip())

                for path, name in aos_remote_components.items():
                    cube_add = False
                    cube_remove = False

                    rel_path = os.path.dirname(relpath(pd.get_cfg(PROGRAM_PATH), path))
                    if cube_add_components and os.path.dirname(rel_path) in cube_add_components:
                        cube_add = True
                    if cube_remove_components and os.path.dirname(rel_path) in cube_remove_components:
                        cube_remove = True

                    json_components_dict[rel_path] = {'name': name, 'cube_add': cube_add, 'cube_remove': cube_remove}

                for path, name in aos_local_components.items():
                    cube_add = False
                    cube_remove = False

                    rel_aos_path = os.path.dirname(relpath(aos_path, path))
                    rel_path = os.path.join(os_name, rel_aos_path).replace(os.path.sep, '/')

                    if cube_add_components and rel_aos_path in cube_add_components:
                        cube_add = True
                    if cube_remove_components and rel_aos_path in cube_remove_components:
                        cube_remove = True

                    json_components_dict[rel_path] = {'name': name, 'cube_add': cube_add, 'cube_remove': cube_remove}
        else:
            cube_add = False
            cube_remove = False
            for path, name in aos_local_components.items():
                json_components_dict[os.path.dirname(relpath(os.path.dirname(aos_path), path))] = {'name': name, 'cube_add': cube_add, 'cube_remove': cube_remove}

        print(json.dumps(json_components_dict, indent=4, sort_keys=True))
    else:
        if cwd_type != 'program':
            print('\nCurrent directory isn\'t AliOS-Things program, list AOS_SDK_PATH components.')
        print("\n                                                      AliOS-Things COMPONENTS                ")
        print('|===================================================================================================================|')

        print('| %-30s | %-80s |' % ('NAME', 'LOCATION'))

        if cwd_type == 'program':
            program_path = Program(os.getcwd()).get_cfg(PROGRAM_PATH)
            for path, name in aos_remote_components.items():
                print('| %-30s | %-80s |' % (name, os.path.dirname(relpath(program_path, path))))
        for path, name in aos_local_components.items():
            print('| %-30s | %-80s |' % (name, os.path.dirname(relpath(os.path.dirname(aos_path), path))))

        print('|===================================================================================================================|')

def get_aos_components(aos_path):
    makefile_dict = {}
    if os.path.isdir(aos_path):
        for (dir_path, dir_names, file_names) in os.walk(aos_path):
            for f in file_names:
                if ('out' not in dir_path) and ('build' not in dir_path) and ('tools/codesync' not in dir_path) and f.endswith('.mk'):
                    makefile_dict[os.path.join(dir_path, f)] = f[:-3]
    else:
        error('Find components dir is empty!')

    aos_components_dict = {}
    for path, name in makefile_dict.items():
        with open(path, 'r') as f:
            s = f.read()
            component_name = re.findall('^\s*NAME\s*:?=\s*(\S+)\s*\n', s)
            if len(component_name) == 1:
                aos_components_dict[path.replace(os.path.sep, '/')] = component_name[0]

    return aos_components_dict

# List command
@cli.command("list", short_help="List installed components")
@click.argument("components", required=False, nargs=-1, metavar="[COMPONENTS...]")
@click.option("--json-output", is_flag=True)
@click.option("-r", "--remote", is_flag=True, help="List remote components")
@click.option("-d", "--showduplicates", is_flag=True, help="show all versions of components")
@click.pass_obj
def list_component(cm, components, json_output, remote, showduplicates):
    #print_component_status(json_output)
    args = []
    if showduplicates:
        args += ["--showduplicates"]

    if components:
        args += components

    # Clean cached data first
    cm.clean()
    output, err = cm.list(remote, *args)
    click.echo(output)

# Install command
@cli.command("install", short_help="Install components")
@click.argument("components", required=True, nargs=-1, metavar="[COMPONENTS...]")
@click.option("-t", "--check-available", is_flag=True, help="Check if component available before installation")
#@click.option("-s", "--silent", is_flag=True, help="Suppress progress reporting")
#@click.option("--interactive", is_flag=True, help="Allow to make a choice for all prompts")
@click.pass_obj
def install_component(cm, components, check_available):
    cm.install(components, check_available)
    os.chdir(os.path.join(cm.project_dir, OS_NAME))
    update_config_in()

# Uninstall command
@cli.command("uninstall", short_help="Uninstall components")
@click.argument("components", required=False, nargs=-1, metavar="[COMPONENTS...]")
@click.option("-a", "--all", is_flag=True, help="Uninstall all components")
@click.option("-f", "--force", is_flag=True, help="Force to uninstall components")
@click.pass_obj
def uninstall_component(cm, components, all, force):
    if all:
        complist = []
        output, err = cm.list(remote=None)
        patten = re.compile(r"(.*\.noarch)\s*(.*)aos\s*installed")
        for line in output.splitlines():
            match = patten.match(line)
            if match:
                complist += [match.group(1)]

        if complist:
            components = complist

    opts = []
    if force:
        opts += ["--nodeps"]

    cm.uninstall(components, *opts)
    os.chdir(os.path.join(cm.project_dir, OS_NAME))
    update_config_in()

# Update command
@cli.command("update", short_help="Update installed components")
@click.argument("components", required=False, nargs=-1, metavar="[COMPONENTS...]")
@click.option("-c", "--only-check", is_flag=True, help="Do not update, only check for new version")
@click.option("--json-output", is_flag=True)
@click.pass_obj
def update_component(cm, components, only_check, json_output):
    if only_check:
        output, err = cm.update(components, True, json_output)
        if output.strip():
            click.echo(output)
    else:
        cm.update(components)
        os.chdir(os.path.join(cm.project_dir, OS_NAME))
        update_config_in()

# Search command
@cli.command("search", short_help="Search for a component")
@click.argument("keyword", required=True, nargs=-1)
@click.option("--json-output", is_flag=True)
@click.option("-n", "--name", multiple=True)
@click.option("-a", "--author", multiple=True)
@click.pass_obj
def search_component(cm, keyword, json_output, **filters):
    cm.search(keyword)

# Show command
@cli.command("show", short_help="Show detailed info about a component")
@click.argument("component", required=False, nargs=-1)
@click.option("--json-output", is_flag=True)
@click.option("-r", "--remote", is_flag=True, help="List remote components")
@click.pass_obj
def show_component(cm, json_output, remote, component):
    cm.show(json_output, remote, component)

# Clean command
@cli.command("clean", short_help="Clean cached component metadata")
@click.pass_obj
def clean_metadata(cm):
    cm.clean()



import re
from util import *
import click
import ConfigParser


# Cfg classed used for handling the config backend
class Cfg(object):
    path = None
    file = ".aos"

    def __init__(self, path):
        self.path = path

    # Sets config value
    def set(self, var, val):
        if not re.match(r'^([\w+-]+)$', var):
            error("%s is invalid config variable name" % var)

        fl = os.path.join(self.path, self.file)
        try:
            with open(fl) as f:
                lines = f.read().splitlines()
        except (IOError, OSError):
            lines = []

        for line in lines:
            m = re.match(r'^([\w+-]+)\=(.*)$', line)
            if m and m.group(1) == var:
                lines.remove(line)

        if not val is None:
            lines += [var + "=" + val]

        try:
            with open(fl, 'w') as f:
                f.write('\n'.join(lines) + '\n')
        except (IOError, OSError):
            warning("Unable to write config file %s" % fl)

    # Gets config value
    def get(self, var, default_val=None):
        fl = os.path.join(self.path, self.file)
        try:
            with open(fl) as f:
                lines = f.read().splitlines()
        except (IOError, OSError):
            lines = []

        for line in lines:
            m = re.match(r'^([\w+-]+)\=(.*)$', line)
            if m and m.group(1) == var:
                return m.group(2)
        return default_val

    # Get all config var/values pairs
    def list(self):
        fl = os.path.join(self.path, self.file)
        try:
            with open(fl) as f:
                lines = f.read().splitlines()
        except (IOError, OSError):
            lines = []

        vars = {}
        for line in lines:
            m = re.match(r'^([\w+-]+)\=(.*)$', line)
            if m and m.group(1) and m.group(1) != 'ROOT':
                vars[m.group(1)] = m.group(2)
        return vars


# Global class used for global config
class Global(object):
    path = os.path.join(os.path.expanduser("~"), '.aos')

    def __init__(self):
        if not os.path.exists(Global.path):
            try:
                os.mkdir(Global.path)
            except (IOError, OSError):
                try:
                    os.mkdir(Global.path)
                except (IOError, OSError):
                    pass

    @staticmethod
    def get_path():
        return Global.path

    @staticmethod
    def get_cfg(*args, **kwargs):
        return Cfg(Global.path).get(*args, **kwargs)

    @staticmethod
    def set_cfg(*args, **kwargs):
        return Cfg(Global.path).set(*args, **kwargs)

    @staticmethod
    def list_cfg(*args, **kwargs):
        return Cfg(Global.path).list(*args, **kwargs)

class Config():
    def __init__(self, conf_file):
        self.conf_file = conf_file
        self.conf = ConfigParser.ConfigParser()
        self.conf.read(self.conf_file)

    def list(self, section=None, name=None):
        if section and name:
            if not section in self.conf.sections():
                error("No section %s found in %s" % (section, self.conf_file))
                return
            if not name in self.conf.options(section):
                error("No option %s found in %s" % (name, self.conf_file))
                return

            click.echo("%s.%s=%s" % (section, name, self.conf.get(section, name)))

        elif section:
            for name in self.conf.options(section):
                click.echo("%s.%s=%s" % (section, name, self.conf.get(section, name)))
        else:
            for section in self.conf.sections():
                for name in self.conf.options(section):
                    click.echo("%s.%s=%s" % (section, name, self.conf.get(section, name)))

    def set(self, section, name, value):
        if not section in self.conf.sections():
            self.conf.add_section(section)

        self.conf.set(section, name, value)

    def clean(self, section):
        if section in self.conf.sections():
            self.conf.remove_section(section)

    def write(self):
        with open (self.conf_file, "w") as f:
            self.conf.write(f)



import contextlib
import os
import sys
import shutil
import stat
import subprocess
import platform
import re

import errno

from aos.constant import *


# Directory navigation
@contextlib.contextmanager
def cd(newdir):
    prevdir = os.getcwd()
    os.chdir(newdir)
    try:
        yield
    finally:
        os.chdir(prevdir)


def relpath(root, path):
    return path[len(root) + 1:]


def staticclass(cls):
    for k, v in cls.__dict__.items():
        if hasattr(v, '__call__') and not k.startswith('__'):
            setattr(cls, k, staticmethod(v))

    return cls


# Logging and output
def log(msg):
    sys.stdout.write(msg)
    sys.stdout.flush()


def message(msg):
    return "[AliOS-Things] %s\n" % msg


def info(msg, level=1):
    if level <= 0 or verbose:
        for line in msg.splitlines():
            log(message(line))


def action(msg):
    for line in msg.splitlines():
        log(message(line))


def warning(msg):
    for line in msg.splitlines():
        sys.stderr.write("[AliOS-Things] WARNING: %s\n" % line)
    sys.stderr.write("---\n")


def error(msg, code=-1):
    for line in msg.splitlines():
        sys.stderr.write("[AliOS-Things] ERROR: %s\n" % line)
    sys.stderr.write("---\n")
    sys.exit(code)


def progress_cursor():
    while True:
        for cursor in '|/-\\':
            yield cursor


progress_spinner = progress_cursor()


def progress():
    sys.stdout.write(progress_spinner.next())
    sys.stdout.flush()
    sys.stdout.write('\b')


# Process execution
class ProcessException(Exception):
    pass


def rmtree_readonly(directory):
    def remove_readonly(func, path, _):
        os.chmod(path, stat.S_IWRITE)
        func(path)

    shutil.rmtree(directory, onerror=remove_readonly)


def popen(command, suppress_error=None, stdin=None, **kwargs):
    # print for debugging
    info('Exec "' + ' '.join(command) + '" in ' + os.getcwd())

    # fix error strings
    try:
        command_line = command.split()
    except:
        command_line = command

    try:
        proc = subprocess.Popen(command, **kwargs)
    except OSError as e:
        if e[0] == errno.ENOENT:
            error(
                "Could not execute \"%s\".\n"
                "Please verify that it's installed and accessible from your current path by executing \"%s\".\n" % (
                command_line[0], command_line[0]), e[0])
        else:
            raise e

    if proc.wait() != 0:
        if not suppress_error:
            raise ProcessException(proc.returncode, command_line[0], ' '.join(command_line), os.getcwd())

    return proc.returncode


def pquery(command, stdin=None, **kwargs):
    if very_verbose:
        info('Query "' + ' '.join(command) + '" in ' + os.getcwd())
    try:
        proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs)
    except OSError as e:
        if e[0] == errno.ENOENT:
            error(
                "Could not execute \"%s\".\n"
                "Please verify that it's installed and accessible from your current path by executing \"%s\".\n" % (
                command[0], command[0]), e[0])
        else:
            raise e

    stdout, _ = proc.communicate(stdin)

    if very_verbose:
        log(str(stdout).strip() + "\n")

    if proc.returncode != 0:
        raise ProcessException(proc.returncode, command[0], ' '.join(command), os.getcwd())

    return stdout

def exec_cmd(command, suppress_error=None, stdin=None, **kwargs):
    """ Run command and return output, errcode """
    info('Exec "' + ' '.join(command) + '" in ' + os.getcwd())

    # fix error strings
    if isinstance(command, str):
        command_line = command.split()
    else:
        command_line = command

    try:
        proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
    except OSError as e:
        if e[0] == errno.ENOENT:
            error(
                "Could not execute \"%s\".\n"
                "Please verify that it's installed and accessible from your current path by executing \"%s\".\n" % (
                command_line[0], command_line[0]), e[0])
        else:
            raise e

    output, err = proc.communicate()
    if proc.wait() != 0:
        if not suppress_error:
            raise ProcessException(proc.returncode, command_line[0], ' '.join(command_line), os.getcwd())

    return output, err

def get_country_code():
    import requests
    import json
    url = "https://geoip-db.com/json"
    try:
        res = requests.get(url, timeout = 5)
        data = json.loads(res.text)
        return data['country_code']
    except Exception:
        return 'CN'

def is_domestic():
    if get_country_code() == 'CN':
        return True
    else:
        return False

def get_host_os():
    host_os = platform.system()
    if host_os == 'Windows':
        host_os = 'Win32'
    elif host_os == 'Linux':
        if platform.machine().endswith('64'):
            bit = '64'
        else:
            bit = '32'
        host_os += bit
    elif host_os == 'Darwin':
        host_os = 'OSX'
    else:
        host_os = None
    return host_os

def get_aos_url():
    """Figure out proper URL for downloading AliOS-Things."""
    if is_domestic():
        aos_url = 'https://gitee.com/alios-things/AliOS-Things.git'
    else:
        aos_url = 'https://github.com/alibaba/AliOS-Things.git'

    return aos_url

def cd_aos_root():
    original_dir = os.getcwd()
    host_os = get_host_os()
    if host_os == 'Win32':
        sys_root = re.compile(r'^[A-Z]{1}:\\$')
    else:
        sys_root = re.compile('^/$')
    while os.path.isdir('./include/aos') == False and os.path.isdir('./kernel/rhino') == False:
        os.chdir('../')
        if sys_root.match(os.getcwd()):
            return 'fail', original_dir
    return 'success', original_dir

def get_aos_project():
    """ Figure out the aos project dir """
    curr_dir = os.getcwd()

    host_os = get_host_os()
    if host_os == 'Win32':
        sys_root = re.compile(r'^[A-Z]{1}:\\$')
    else:
        sys_root = re.compile('^/$')

    while not os.path.isfile(os.path.join(curr_dir, ".aos", OS_CONFIG)):
        curr_dir = os.path.abspath(os.path.join(curr_dir, "../"))

        if sys_root.match(curr_dir):
            return None

    return curr_dir

def which(program):
    if platform.system() == 'Windows' and program.endswith('.exe') == False:
        program += '.exe'

    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file
    return None

def cmd_version_match(cmd, version):
    cmd = which(cmd)
    if cmd == None:
        return False
    if version == 'all':
        return True
    match = False
    try:
        ret = subprocess.check_output([cmd, '-v'], stderr=subprocess.STDOUT)
    except:
        return match
    lines = ret.split('\n')
    for line in lines:
        if version in line:
            match = True
            break;
    return match

def get_config_value(keyword, config_file=None):
    """ Get value of keyword from config file """
    value = None
    if not config_file:
        config_file = '.config'

    if not os.path.isfile(config_file):
        return value

    with open(config_file) as f:
        for line in f.readlines():
            match = re.match(r'^([\w+-]+)\=(.*)$', line)
            if match and match.group(1) == keyword:
                value = match.group(2).replace('"','')

    return value

def get_board_dir(board):
    """ Get board dir that include version number """
    board_dir = None
    for root, dirs, files in os.walk('./board/'):
        for tmpdir in dirs:
            if "%s-" % board in tmpdir:
                board_dir = tmpdir

    return board_dir

def update_config_in():
    """ Call build/scripts/gen_configin.py """
    ret = 0
    gen_config_script = "build/scripts/gen_configin.py"
    if os.path.isfile(gen_config_script):
        ret = popen(gen_config_script)

    return ret

def check_url(url):
    """ Check if the url available """
    ret = 0
    code = 404

    try:
        from urllib2 import urlopen
    except:
        from urllib.request import urlopen

    try:
        resp = urlopen(url)
        code = resp.getcode()
    except:
        pass

    if code != 200:
        ret = 1

    return ret



import os, sys
import shutil
import json, re
import click
import platform

from aos.util import popen, exec_cmd, get_aos_project, error
from aos.constant import *
import ConfigParser

class Rpm():
    """ Wrapper for rpm commands """

    def __init__(self, dbpath):
        self.dbpath = dbpath
        self.rpm = "rpm --dbpath %s" % dbpath

    def install(self, install_dir, rpm_dir):
        """ Install rpms to install_dir """

        cmd = "%s -ivh --relocate /=%s %s/*.rpm" % (self.rpm, install_dir, rpm_dir)
        popen(cmd.split())

    def uninstall(self, compname, *opts):
        """ Remove installed rpms """
        args = []
        args += compname
        if opts:
            args += opts

        cmd = "%s -e %s" % (self.rpm, " ".join(args))
        popen(cmd.split())

    def query(self, *opts):
        """ Run rpm query """
        cmd = "%s -q" % (self.rpm)
        args = []
        if opts:
            args += opts

            output, err = exec_cmd(cmd.split() + args)
            return output, err

class Yum():
    """ Wrapper for yum commands """

    def __init__(self, yumconf):
        self.conf = yumconf
        self.yum = "yumx -c %s" % yumconf

    def init_conf(self, baseurl=None):
        """ Create customized yum.conf """

        cf = ConfigParser.ConfigParser()
        cf.add_section("main")
        cf.set("main", "cachedir", "/var/cache/yum/aos")
        cf.set("main", "keepcache", "0")
        cf.set("main", "debuglevel", "2")
        cf.set("main", "logfile", "/var/log/yum.log")
        cf.set("main", "exactarch", "0")
        cf.set("main", "obsoletes", "1")
        cf.set("main", "gpgcheck", "0")
        cf.set("main", "plugins", "0")
        cf.set("main", "installonly_limit", "5")
        cf.set("main", "reposdir", "")
        cf.set("main", "installroot", os.path.dirname(self.conf))

        reponame = "AliOS_Things"
        cf.add_section(reponame)
        cf.set(reponame, "name", "Server")
        if baseurl:
            cf.set(reponame, "baseurl", baseurl)
        else:
            cf.set(reponame, "baseurl", OS_REPO)

        cf.set(reponame, "enable", "1")

        cf.write(open(self.conf, "w"))

    def clean(self):
        """ Clean cached metadata """
        cmd = "%s clean all" % self.yum
        exec_cmd(cmd.split())

    def list(self, remote, *opts):
        """ List installed/remote components """

        cmd = "%s list" % self.yum
        args = []
        suppress_error = False
        if not remote:
            args = ["installed"]

        for opt in opts:
            if "--suppress_error" == opt:
                suppress_error = True
                continue
            args += [ opt ]

        output, err = exec_cmd(cmd.split() + args, suppress_error)
        return output, err

    def search(self, keyword):
        """ Search components with keyword """

        cmd = "%s search %s" % (self.yum, " ".join(keyword))
        popen(cmd.split())

    def install(self, args=None):
        """ TODO: Install components """

        cmd = "%s install" % self.yum
        if args:
            popen(cmd.split() + args)
        else:
            click.echo("Nothing to install ...")

    def uninstall(self, args):
        """ TODO: Remove installed components """

        cmd = "%s remove %s" % (self.yum, " ".join(args))
        popen(cmd.split())

    def update(self, args=None, only_check=None, json_output=None):
        """ TODO: Update installed components """

        if only_check:
            cmd = "%s check-update" % self.yum
            # suppress exit code 100 for yum check-update:
            # 100 if there are packages available for update
            suppress_error = True
            if args:
                output, err = exec_cmd(cmd.split() + list(args), suppress_error)
            else:
                output, err = exec_cmd(cmd.split(), suppress_error)
            return output, err
        else:
            cmd = "%s update" % self.yum

            if args:
                popen(cmd.split() + list(args))
            else:
                popen(cmd.split())

    def download(self, download_dir, args=None):
        """ Download rpms to download_dir """

        cmd = "%s install --downloadonly --downloaddir=%s" % (self.yum, download_dir)
        if args:
            popen(cmd.split() + list(args))
        else:
           click.echo("Nothing to download ...")

    def show(self, json_format=None, remote=None, args=None):
        """ Show components information """

        cmd = "%s info" % self.yum
        if not remote and not args:
            args = ["installed"]

        if not json_format:
            popen(cmd.split() + list(args))
        else:
            json_output = []
            patten = re.compile(r"^(\w+)\s*:\s*(.*)")
            output, ret = exec_cmd(cmd.split() + list(args))
            for line in output.splitlines():
                match = patten.match(line)
                if match:
                    if "Name" == match.group(1):
                        tmp = { "name": match.group(2) }
                        curr_field = None
                    elif "Version" == match.group(1):
                        tmp["version"] = match.group(2)
                    elif "Summary" == match.group(1):
                        tmp["summary"] = match.group(2)
                        curr_field = tmp["summary"]
                    elif "Description" == match.group(1):
                        tmp["description"] = match.group(2)
                        curr_field = tmp["description"]
                    elif not match.group(1) and match.group(2):
                        print match.group(1)
                        print match.group(2)
                        curr_field += " " + match.group(2)
                    else:
                        pass

                if not line.strip():
                    json_output.append(tmp)

            # Get component dependency
            for item in json_output:
                comp_name = item["name"]
                item["depends"] = []
                patten = re.compile(r"\s*(dependency|provider):\s*(.*)")
                cmd = "%s deplist %s" % (self.yum, comp_name)
                output, ret = exec_cmd(cmd.split())
                for line in output.splitlines():
                    match = patten.match(line)
                    if match:
                        if "dependency" == match.group(1):
                            tmp = { "name": match.group(2) }
                        elif "provider" == match.group(1):
                            subpatten = re.compile(r".*.noarch\s*(.*)-r\d*.aos")
                            submatch = subpatten.match(match.group(2))
                            if submatch:
                                tmp["min_version"] = submatch.group(1)
                                tmp["max_version"] = ""
                                item["depends"].append(tmp)
                        else:
                            pass

            click.echo(json.dumps(json_output, sort_keys=True, indent=4))

class CompManager():
    """ Component Manager bases on Rpm/Yum infrastructure """

    def __init__(self, baseurl=None):
        project_dir = get_aos_project()
        if project_dir:
            self.project_dir = project_dir
        else:
            error("Can't find AliOS Things Project directory ...")

        if "Ubuntu" in platform.dist():
            dbpath = os.path.expanduser('~') + "/.rpmdb"
        else:
            dbpath = "/var/lib/rpm"

        self.rpmdb = "%s/%s%s" % (project_dir, ".aos", dbpath)
        self.rpm = Rpm(self.rpmdb)

        self.yumconf = os.path.join(project_dir, ".aos", "yum.conf")
        self.yum = Yum(self.yumconf)
        if not os.path.isfile(self.yumconf):
            self.yum.init_conf(baseurl)

    def install(self, comp_list, check_available=None):
        """ Install components.

        Workaroud: download rpms and then install them
        TODO:      install components directly
        """

        click.echo("Install Component: %s" % " ".join(comp_list))

        download_dir = os.path.join(self.project_dir, ".aos", "downloads")
        install_dir = os.path.join(self.project_dir, OS_NAME)

        # check available for compname
        if check_available:
            for compname in comp_list:
                opts = ["--suppress_error"]
                opts += [ compname ]
                remote = True

                output, err = self.list(remote, *opts)
                if err:
                    error("No matching component found: %s" % compname)
                    sys.exit(1)
                else:
                    click.echo("Component %s available" % compname)

        # cleanup download_dir first
        shutil.rmtree(download_dir)
        os.mkdir(download_dir)

        self.yum.download(download_dir, comp_list)
        if os.listdir(download_dir):
            self.rpm.install(install_dir, download_dir)

    def uninstall(self, compname, *opts):
        """ Remove installed components """

        click.echo("Uninstall component: %s" % " ".join(compname))
        self.rpm.uninstall(compname, *opts)
    
    def update(self, compname=None, only_check=None, json_output=None):
        """ Update installed components """

        if compname:
            click.echo("Update component: %s" % " ".join(compname))
        else:
            click.echo("Update all installed components")

        if only_check:
            output, err = self.yum.update(compname, only_check, json_output)
            return output, err
        else:
            self.yum.update(compname, only_check, json_output)

    def search(self, compname):
        """ Search remote components """

        click.echo("Search component: %s" % " ".join(compname))
        self.yum.search(compname)

    def list(self, remote, *opts):
        """ List installed/remote components """

        output, err = self.yum.list(remote, *opts)
        return output, err

    def show(self, json_format, remote, compname=None):
        """ Show installed/remote components information """

        self.yum.show(json_format, remote, compname)

    def clean(self):
        """ Clean cached metadata """
        self.yum.clean()

    def rpmquery(self, *opts):
        """ Run rpm query """
        output, err = self.rpm.query(*opts)
        return output, err



#!/usr/bin/env python

import sys, os

DEFAULT_SERVER = '118.31.76.36'
DEFAULT_PORT = 2000

def print_usage():
    print "Usage: {0} terminal/client [-s xxx.xxx.xx.xx] [-p xxxxx]\n".format(sys.argv[0])

def try_install_curses():
    try:
        import curses
        return 'success'
    except:
        pass

    if os.name != 'nt':
        return 'fail'
    print 'installing dependent package: curses ...'
    import subprocess, urllib
    try:
        curses_32bit_url = 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/curses-2.2-cp27-none-win32.whl'
        curses_64bit_url = 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/curses-2.2-cp27-none-win_amd64.whl'
        urllib.urlretrieve(curses_32bit_url, 'curses-2.2-cp27-none-win32.whl')
        urllib.urlretrieve(curses_64bit_url, 'curses-2.2-cp27-none-win_amd64.whl')
        subprocess.call(['pip', 'install', 'wheel'])
        subprocess.call(['pip', 'install', 'curses-2.2-cp27-none-win32.whl'])
        subprocess.call(['pip', 'install', 'curses-2.2-cp27-none-win_amd64.whl'])
    except:
        pass
    finally:
        if os.path.isfile('curses-2.2-cp27-none-win32.whl'):
            os.remove('curses-2.2-cp27-none-win32.whl')
        if os.path.isfile('curses-2.2-cp27-none-win_amd64.whl'):
            os.remove('curses-2.2-cp27-none-win_amd64.whl')

    try:
        import curses
        print 'succeed'
        return 'success'
    except:
        print 'failed'
        return 'fail'

def main():
    if len(sys.argv) < 2:
        print_usage()
        sys.exit(1)

    host_name = DEFAULT_SERVER
    host_port = DEFAULT_PORT

    mode = sys.argv[1]
    i = 2; arg_num = len(sys.argv)
    while i < arg_num:
        if sys.argv[i] == '-s' and (i+1) < arg_num:
            host_name = sys.argv[i+1]
            i += 1
        elif sys.argv[i] == '-p' and (i+1) < arg_num:
            try:
                host_port = int(sys.argv[i+1])
            except:
                print "error: valid port '{0}'".format(sys.argv[i+1])
                sys.exit(1)
            i += 1
        else:
            print "error: invalid argument '{0}'".format(' '.join(sys.argv[1:]))
            print_usage()
            sys.exit(1)
        i += 1

    if os.name == 'posix':
        tmpfile_folder = '/tmp/'
    elif os.name == 'nt':
        tmpfile_folder = os.path.expanduser('~') + '\\'

    if mode == "client":
        from client import Client
        if host_name == None:
            print_usage()
            sys.exit(1)

        tmpfile = tmpfile_folder + '.testbed_client'
        if os.path.exists(tmpfile):
            print "An udevice center client is already running"
            sys.exit(1)

        open(tmpfile, 'a').close()
        client = Client()
        client.client_func(host_name, host_port)
        os.remove(tmpfile)
    elif mode == "terminal":
        if try_install_curses() == 'fail':
            print "error: unable to install python curses module"
            sys.exit(1)
        from terminal import Terminal
        if host_name == None:
            print_usage()
            sys.exit(1)
        terminal = Terminal()
        terminal.terminal_func(host_name, host_port)
    else:
        print_usage()
    sys.exit(0)

if __name__ == '__main__':
    main()



import os, sys
import click

from aos.util import log, error, popen
from aos.config import *

# Make command
@click.command("upload", short_help="Upload aos image")
@click.argument("target", required=False, nargs=-1)
@click.option("--work-path", "-w", help="Alternative work path if aos_sdk_path is unavailable")
@click.option("--binaries-dir", "-b", help="Dir to store upload binaries")
def cli(target, work_path, binaries_dir):
    """ Upload aos image to target platform. """
    from aos import __version__
    log('aos-cube version: %s\n' % __version__)

    upload_target = ' '.join(target)

    if work_path:
        if os.path.isdir(work_path):
            aos_path = work_path
        else:
            error("Can't find dir %s" % work_path)
    else:
        #cd to aos root_dir
        ret, original_dir = cd_aos_root()
        if ret != 'success':
            log("[INFO]: Not in AliOS-Things source code directory")
            log("[INFO]: Current directory is: '%s'\n" % original_dir)
            if os.path.isdir(original_dir):
                os.chdir(original_dir)
            aos_path = Global().get_cfg(AOS_SDK_PATH)
            if aos_path == None:
                error("[ERRO]: Not in aos_sdk_path, aos_sdk unavailable as well!")
            else:
                log("[INFO]: Config Loading OK, use '%s' as sdk path\n" % aos_path)
        else:
            aos_path = os.getcwd()
            log("[INFO]: Currently in aos_sdk_path: '%s'\n" % os.getcwd())

    # read app & board from .config
    if upload_target == '':
        # check AliOS Things version
        if os.path.exists(os.path.join(aos_path, 'build', 'Config.in')) == False:
            error('Target invalid')

        board = None
        app = None
        log("[INFO]: Not set target, read target from .config\n")

        config_file = os.path.join(aos_path, '.config')
        if os.path.exists(config_file) == False:
            error('Config file NOT EXIST: %s\n' % config_file)

        board = get_config_value('AOS_BUILD_BOARD', config_file)
        app = get_config_value('AOS_BUILD_APP', config_file)

        if not app and not board:
            error('None target in %s\n' % config_file)
        upload_target = '%s@%s' % (app, board)

    elif '@' not in upload_target or len(upload_target.split('@')) != 2:
        error('Target invalid')
        return

    extra_args = []
    extra_args += ['WORKPATH=' + aos_path]

    if binaries_dir and not os.path.isdir(binaries_dir):
        error("Can't find dir %s" % binaries_dir)

    extra_args = []
    if work_path:
        extra_args += ['WORKPATH=' + work_path]
    if binaries_dir:
        extra_args += ['BINDIR=' + binaries_dir]

    click.secho("[INFO]: Target: %s\n" % upload_target, fg="green")
    _call_scons_target(upload_target, 'upload', aos_path, extra_args)

def _call_scons_target(target, command, aos_path, extra_args=None):
    """ Call scons to run specific command """
    sconscript = os.path.join(aos_path, 'ucube.py')
    if not os.path.exists(sconscript):
        sconscript = os.path.join(aos_path, 'build/ucube.py')

    make_args = ['scons -f ' + sconscript + ' COMMAND=' + command]

    target_find = False

    if '@' in target and not target_find:
        targets = target.split('@')
        if len(targets) < 2:
            error('Must special app and board when build aos')

        app = 'APPLICATION='+targets[0]
        board = 'BOARD=' + targets[1]
        make_args.append(app)
        make_args.append(board)

        if len(targets) == 3:
            build_type = 'TYPE=' + targets[2]
            make_args.append(build_type)

        target_find = True

    if extra_args:
        make_args += extra_args

    ret = popen(' '.join(make_args), shell=True, cwd=os.getcwd())

    return ret



import ctypes

from aos.config import *
from aos.util import *
from aos.repo import *


# Program class, acts code base root
class Program(object):
    path = None
    name = None
    is_cwd = False
    is_repo = False
    is_classic = False
    build_dir = "BUILD"

    def __init__(self, path=None, print_warning=False):
        path = os.path.abspath(path or os.getcwd())
        self.path = path
        self.is_cwd = True

        while cd(path):
            tpath = path
            if os.path.isfile(os.path.join(path, Cfg.file)):
                self.path = path
                self.is_cwd = False
                break

            path = os.path.split(path)[0]
            if tpath == path:       # Reached root.
                break

        self.name = os.path.basename(self.path)
        self.is_classic = os.path.isfile(os.path.join(self.path, 'aos.bld'))

        # is_cwd flag indicates that current dir is assumed to be root, not root repo
        if self.is_cwd and print_warning:
            warning(
                "Could not find aos program in current path \"%s\".\n"
                "You can fix this by calling \"aos new .\" in the root of your program." % self.path)

    def get_cfg(self, *args, **kwargs):
        return Cfg(self.path).get(*args, **kwargs) or Global().get_cfg(*args, **kwargs)

    def set_cfg(self, *args, **kwargs):
        return Cfg(self.path).set(*args, **kwargs)

    def list_cfg(self, *args, **kwargs):
        return Cfg(self.path).list(*args, **kwargs)

    def set_root(self):
        return self.set_cfg('ROOT', '.')

    def unset_root(self, path=None):
        fl = os.path.join(path or self.path, Cfg.file)
        if os.path.isfile(fl):
            os.remove(fl)

    # Gets aos dir (unified)
    def get_os_dir(self):
        if os.path.isdir(os.path.join(self.path, 'aos')):
            return os.path.join(self.path, 'aos')
        elif self.name == 'aos':
            return self.path
        else:
            return None

    def get_aoslib_dir(self):
        if os.path.isdir(os.path.join(self.path, 'aos')):
            return os.path.join(self.path, 'aos')
        else:
            return None

    # Gets aos tools dir (unified)
    def get_tools_dir(self):
        paths = []
        # aos dir identified and tools is a sub dir
        aos_os_path = self.get_os_dir()
        if aos_os_path:
            paths.append([aos_os_path, 'tools'])
            paths.append([aos_os_path, 'core', 'tools'])
        # aos not identified but tools found under cwd/tools
        paths.append([self.path, 'tools'])
        paths.append([self.path, 'core', 'tools'])
        # aos Classic deployed tools
        paths.append([self.path, '.temp', 'tools'])

        return self._find_file_paths(paths, 'make.py')

    def get_requirements(self):
        paths = []
        aos_os_path = self.get_os_dir()
        if aos_os_path:
            paths.append([aos_os_path, 'tools'])
            paths.append([aos_os_path])
        # aos not identified but tools found under cwd/tools
        paths.append([self.path, 'tools'])
        # aos Classic deployed tools
        paths.append([self.path, '.temp', 'tools'])
        # current dir
        paths.append([self.path])

        return self._find_file_paths(paths, 'requirements.txt')

    def _find_file_paths(self, paths, fl):
        for p in paths:
            path = os.path.join(*p)
            if os.path.isdir(path) and os.path.isfile(os.path.join(path, fl)):
                return os.path.join(path)
        return None

    def check_requirements(self, show_warning=False):
        req_path = self.get_requirements() or self.path
        req_file = 'requirements.txt'
        missing = []
        try:
            with open(os.path.join(req_path, req_file), 'r') as f:
                import pip
                installed_packages = [re.sub(r'-', '_', package.project_name.lower()) for package in pip.get_installed_distributions(local_only=True)]
                for line in f.read().splitlines():
                    pkg = re.sub(r'-', '_', re.sub(r'^([\w-]+).*$', r'\1', line).lower())
                    if not pkg in installed_packages:
                        missing.append(pkg)

                if missing and install_requirements:
                    try:
                        action("Auto-installing missing Python modules...")
                        pquery(['pip', 'install', '-q', '-r', os.path.join(req_path, req_file)])
                        missing = []
                    except ProcessException:
                        warning("Unable to auto-install required Python modules.")

        except (IOError, ImportError, OSError):
            pass

        if missing:
            err = (
                "-----------------------------------------------------------------\n"
                "The aos OS tools in this program require the following Python modules: %s\n"
                "You can install all missing modules by running \"pip install -r %s\" in \"%s\"" % (', '.join(missing), req_file, req_path))
            if os.name == 'posix':
                err += "\nOn Posix systems (Linux, Mac, etc) you might have to switch to superuser account or use \"sudo\""

            if show_warning:
                warning(err)
            else:
                error(err, 1)


    # Routines after cloning aos
    def post_action(self):
        aos_tools_path = self.get_tools_dir()

        if not aos_tools_path and (self.is_classic or os.path.exists(os.path.join(self.path, Cfg.file))):
            self.add_tools(os.path.join(self.path, '.temp'))
            aos_tools_path = self.get_tools_dir()

        if not aos_tools_path:
            warning("Cannot find the aos tools directory in \"%s\"" % self.path)
            return False

        if (not os.path.isfile(os.path.join(self.path, 'aos_settings.py')) and
                os.path.isfile(os.path.join(aos_tools_path, 'default_settings.py'))):
            shutil.copy(os.path.join(aos_tools_path, 'default_settings.py'), os.path.join(self.path, 'aos_settings.py'))

        self.check_requirements(True)

    def add_tools(self, path):
        if not os.path.exists(path):
            os.mkdir(path)
        with cd(path):
            tools_dir = 'tools'
            if not os.path.exists(tools_dir):
                try:
                    action("Couldn't find build tools in your program. Downloading the aos 2.0 SDK tools...")
                    repo = Repo.fromurl(aos_sdk_tools_url)
                    repo.clone(aos_sdk_tools_url, tools_dir)
                except Exception:
                    if os.path.exists(tools_dir):
                        rmtree_readonly(tools_dir)
                    error("An error occurred while cloning the aos SDK tools from \"%s\"" % aos_sdk_tools_url)

    def update_tools(self, path):
        tools_dir = 'tools'
        if os.path.exists(os.path.join(path, tools_dir)):
            with cd(os.path.join(path, tools_dir)):
                try:
                    action("Updating the aos 2.0 SDK tools...")
                    repo = Repo.fromrepo()
                    repo.update()
                except Exception:
                    error("An error occurred while update the aos SDK tools from \"%s\"" % aos_sdk_tools_url)

    def get_tools(self):
        aos_tools_path = self.get_tools_dir()
        if not aos_tools_path:
            error('The aos tools were not found in "%s". \nRun `aos deploy` to install dependencies and tools. ' % self.path, -1)
        return aos_tools_path

    def get_env(self):
        env = os.environ.copy()
        env['PYTHONPATH'] = os.path.abspath(self.path)
        compilers = ['ARM', 'GCC_ARM', 'IAR']
        for c in compilers:
            if self.get_cfg(c+'_PATH'):
                env['aos_'+c+'_PATH'] = self.get_cfg(c+'_PATH')

        return env

    def get_target(self, target=None):
        target_cfg = self.get_cfg('TARGET')
        target = target if target else target_cfg

        if target and (target.lower() == 'detect' or target.lower() == 'auto'):
            targets = self.get_detected_targets()
            if targets == False:
                error("The target detection requires that the 'aos-ls' python module is installed.\nYou can install aos-ls by running 'pip install aos-ls'.")
            elif len(targets) > 1:
                error("Multiple targets were detected.\nOnly 1 target board should be connected to your system when you use the '-m auto' switch.")
            elif len(targets) == 0:
                error("No targets were detected.\nPlease make sure a target board is connected to this system.")
            else:
                action("Detected \"%s\" connected to \"%s\" and using com port \"%s\"" % (targets[0]['name'], targets[0]['mount'], targets[0]['serial']))
                target = targets[0]['name']

        if target is None:
            error("Please specify target using the -m switch or set default target using command 'aos target'", 1)
        return target

    def get_toolchain(self, toolchain=None):
        toolchain_cfg = self.get_cfg('TOOLCHAIN')
        tchain = toolchain if toolchain else toolchain_cfg
        if tchain is None:
            error("Please specify toolchain using the -t switch or set default toolchain using command 'aos toolchain'", 1)
        return tchain

    def set_defaults(self, target=None, toolchain=None):
        if target and not self.get_cfg('TARGET'):
            self.set_cfg('TARGET', target)
        if toolchain and not self.get_cfg('TOOLCHAIN'):
            self.set_cfg('TOOLCHAIN', toolchain)

    def get_macros(self):
        macros = []
        if os.path.isfile('MACROS.txt'):
            with open('MACROS.txt') as f:
                macros = f.read().splitlines()
        return macros


    def ignore_build_dir(self):
        build_path = os.path.join(self.path, self.build_dir)
        if not os.path.exists(build_path):
            os.mkdir(build_path)
        if not os.path.exists(os.path.join(build_path, '.aosignore')):
            try:
                with open(os.path.join(build_path, '.aosignore'), 'w') as f:
                    f.write('*\n')
            except IOError:
                error("Unable to write build ignore file in \"%s\"" % os.path.join(build_path, '.aosignore'), 1)

    def get_detected_targets(self):
        targets = []
        try:
            import aos_lstools
            oldError = None
            if os.name == 'nt':
                oldError = ctypes.windll.kernel32.SetErrorMode(1) # Disable Windows error box temporarily. note that SEM_FAILCRITICALERRORS = 1
            aoss = aos_lstools.create()
            detect_muts_list = aoss.list_aoss()
            if os.name == 'nt':
                ctypes.windll.kernel32.SetErrorMode(oldError)

            for mut in detect_muts_list:
                targets.append({
                    'id': mut['target_id'], 'name': mut['platform_name'],
                    'mount': mut['mount_point'], 'serial': mut['serial_port']
                })
        except (IOError, ImportError, OSError):
            return False

        return targets



import sys
import json
import click

from serial.tools.list_ports import comports

# Make command
@click.command("devices", short_help="List devices on serial ports")
def cli():
    """ List devices on serial ports """
    arr = []
    for p in comports():
        j = json.dumps(p.__dict__)
        arr.append(json.loads(j))
    print(json.dumps(arr, indent = 4))
    sys.exit(0)



import os, sys
import traceback
import click
import errno

from aos import __version__, __email__
from aos.util import error, ProcessException

class AOSCLI(click.MultiCommand):
    def list_commands(self, ctx):
        cmds = []
        aosdir = os.path.dirname(os.path.abspath(__file__))
        for cmdfile in os.listdir(os.path.join(aosdir, "commands")):
            if cmdfile == "__init__.py":
                continue

            if cmdfile.endswith(".py"):
                valid_cmd = 1
                # comp depends on rpm program and rpm module ...
                if cmdfile == "comp.py" or cmdfile == "init.py":
                    try:
                        import rpm
                        import yumx
                    except:
                        valid_cmd = 0

                # ota depends paho-mqtt ...
                elif cmdfile == "ota.py":
                    try:
                        import paho.mqtt.client
                    except:
                        valid_cmd = 0

                if valid_cmd:
                    cmds.append(cmdfile[:-3])

        cmds.sort()
        return cmds

    def get_command(self, ctx, cmd_name):
        mod = None
        try:
            mod = __import__("aos.commands." + cmd_name, None, None,
                             ["cli"])
        except ImportError:
            try:
                return self._handle_obsolate_command(cmd_name)
            except AttributeError:
                raise click.UsageError('No such command "%s"' % cmd_name, ctx)

        return mod.cli

    @staticmethod
    def _handle_obsolate_command(name):
        obsolates = ["setup", "new", "ls", "import", "add", "rm", "deploy", "codes", "publish",
                     "update", "sync", "status", "makelib",]

        if name in obsolates:
            raise click.UsageError('Command "%s" is deprecated since 0.3.x ...\n\nIf you still need such command, please install aos-cube 0.2.x with:\n\n  $ pip install aos-cube==0.2.66\n\nAny feedback please contact with: %s\n' % (name, __email__))

        if name == "scons":
            from aos.commands import make
            return make.cli

        raise AttributeError()

@click.command(
    cls=AOSCLI,
    context_settings=dict(help_option_names=["-h", "--help"]))
@click.option("--verbose", "-v", is_flag=True, help="Verbose diagnostic output")
@click.option("--very-verbose", "-vv", is_flag=True, help="Very verbose diagnostic output")
@click.version_option(__version__, message="%(version)s")
@click.pass_context
def cli(ctx, verbose, very_verbose):
    pass

def main():
    reload(sys)
    sys.setdefaultencoding('UTF8')

    ret = 0
    try:
        cli(None)
    except ProcessException as e:
        error(
            "\"%s\" returned error code %d.\n"
            "Command \"%s\" in \"%s\"" % (e[1], e[0], e[2], e[3]), e[0])
    except OSError as e:
        traceback.print_exc()
        if e[0] == errno.ENOENT:
            error(
                "Could not detect one of the command-line tools.\n"
                "You could retry the last command with \"-v\" flag for verbose output\n", e[0])
        else:
            error('OS Error: %s' % e[1], e[0])
    except KeyboardInterrupt as e:
        info('User aborted!', -1)
        ret = 255
    except Exception as e:
        traceback.print_exc(file=sys.stdout)
        error("Unknown Error: %s" % e, 255)

    return ret

if __name__ == "__main__":
    sys.exit(main())



import os, sys
import shutil
import hashlib
import json
import threading
import time
import progressbar
import requests
import click
import paho.mqtt.client as mqtt
import base64

from aos.constant import OTA_SERVER, OTA_EMQ_PORT, OTA_WEBSERVER_PORT, OTA_UDEBUG_LIB
from aos.util import log, error, cd_aos_root, is_domestic, popen
from aos.program import *

first_progress = 0
bar = None
timer = {}

# ota command
@click.command("ota", short_help="upload firmware to board over ota, e.g. aos ota helloworld@starterkit")
@click.argument("targets", required=False, nargs=-1, metavar="[TARGETS...]")
@click.option("-d", "--device-name", prompt=True, metavar="[DEVICENAME]")
@click.option("-p", "--product-key", prompt=True, metavar="[PRODUCTKEY]")
@click.option("-f", "--upload-file", default="none", metavar="[FILE]")

def cli(targets, device_name, product_key, upload_file):
    """ upload firmware to board over ota """
    from aos import __version__
    click.echo("aos-cube version: %s" % __version__)
    click.echo("deviceName: %s" % device_name)
    click.echo("productKey: %s" % product_key)

    ota_target = ' '.join(targets)

    udebug_lib = None
    if upload_file != "none":
        # check file type (must be *.bin)
        if not upload_file.endswith(".bin"):
            error("file must be *.bin")

        click.echo("You specified the upload file: %s\n" % upload_file)

        if os.path.isfile(upload_file) == False:
            error("Uploading file not exist")
    else:
        #cd to aos root_dir
        ret, original_dir = cd_aos_root()
        if ret != 'success':
            log("[INFO]: Not in AliOS-Things source code directory")
            log("[INFO]: Current directory is: '%s'\n" % original_dir)
            if os.path.isdir(original_dir):
                os.chdir(original_dir)
            aos_path = Global().get_cfg(AOS_SDK_PATH)
            if aos_path == None:
                error("[ERRO]: Not in aos_sdk_path, aos_sdk unavailable as well!")
            else:
                log("[INFO]: Config Loading OK, use '%s' as sdk path\n" % aos_path)
        else:
            aos_path = os.getcwd()
            log("[INFO]: Currently in aos_sdk_path: '%s'\n" % os.getcwd())

        # read app & board from .config
        if ota_target == '':
            # check AliOS Things version
            if os.path.exists(os.path.join(aos_path, 'build', 'Config.in')) == False:
                error('Target invalid')

            board = None
            app = None
            log("[INFO]: Not set target, read target from .config\n")

            config_file = os.path.join(aos_path, '.config')
            if os.path.exists(config_file) == False:
                error('Config file NOT EXIST: %s\n' % config_file)

            board = get_config_value('AOS_BUILD_BOARD', config_file)
            app = get_config_value('AOS_BUILD_APP', config_file)

            if not app and not board:
                error('None target in %s\n' % config_file)
            ota_target = '%s@%s' % (app, board)

        elif '@' not in ota_target or len(ota_target.split('@')) != 2:
            error('Target invalid')
            return

        click.secho("[INFO]: Target: %s\n" % ota_target, fg="green")

        _ota_bin = os.path.join(aos_path, "out", "%s" % ota_target, "binary", "%s.ota.bin" % ota_target)
        _bin = os.path.join(aos_path, "out", "%s" % ota_target, "binary", "%s.bin" % ota_target)

        # check *.bin or *.ota.bin
        if os.path.isfile(_ota_bin) == True:
            upload_file = _ota_bin
        elif os.path.isfile(_bin) == True:
            upload_file = _bin
        else:
            error("Uploading file not exist(%s.bin or %s.ota.bin)" % (ota_target, ota_target))

        click.echo("[INFO]: Get Uploading file: %s" % upload_file)
        udebug_lib = os.path.join(aos_path, "out", "%s" % ota_target, "libraries", "%s" % OTA_UDEBUG_LIB)
        # check is binary including udebug components
        if os.path.isfile(udebug_lib) == False:
            log("[WARN]: Uploading binary seems not including udev components(%s not exist), maybe you can't upload firmware over ota next time\n" % udebug_lib)
            if not click.confirm(click.style('Do you want to continue?', fg="red"), abort=True):
                error("Cancel upload")
    check_device_online(device_name, product_key)
    return ota_upload(upload_file, device_name, product_key)

# upload binary file to oss for ota download
def ota_upload(upload_file, device_name, product_key):
    try:
        (url, size, md5) = _upload(upload_file, device_name, product_key)
        payload = {
            "url": url,
            "size": size,
            "md5": md5
        }

        mqtt_operation(device_name, product_key, json.dumps(payload))
    except KeyboardInterrupt as e:
        click.secho("User aborted!", fg="red")
        timer.cancel()
        return
    except Exception as e:
        # traceback.print_exc(file=sys.stdout)
        error("Unknown Error: %s" % e)
        return -1

# check device online/offline
def check_device_online(device_name, product_key):
    try:
        r = requests.get("http://%s:%d/client?deviceName=%s&productKey=%s" % (OTA_SERVER, OTA_WEBSERVER_PORT, device_name, product_key))
        if r.status_code != 200:
            error("can not check device online/offline")

        r = r.json()
        if len(r['result']['objects']) == 0:
            click.secho("ERROR: device: %s.%s offline" % (device_name, product_key), fg="red")
            sys.exit(-1)
        click.secho("device: %s.%s online" % (device_name, product_key), fg="green")

    except Exception as e:
        error("Unknown Error: %s" % e)

# get file md5sum
def _get_file_md5(filename):
    _FILE_SLIM = (10*1024*1024) # 100MB
    hmd5 = hashlib.md5()
    f_size = os.stat(filename).st_size
    with open(filename,"rb") as fp:
        if f_size>_FILE_SLIM:
            while(f_size>_FILE_SLIM):
                hmd5.update(fp.read(_FILE_SLIM))
                f_size/=_FILE_SLIM
            if(f_size>0) and (f_size<=_FILE_SLIM):
                hmd5.update(fp.read())
        else:
            hmd5.update(fp.read())
        return hmd5.hexdigest()

# upload binary to OSS
def _upload(file, dn, pk):
    upload_file = os.path.abspath(file)
    size = os.path.getsize(upload_file)
    md5 = _get_file_md5(upload_file).upper()
    #print "local file md5: " + md5

    try:
        data = {
            "deviceName": dn,
            "productKey": pk
        }
        files = {
            "field1": open(upload_file, "rb")
        }
        click.echo("uploading file: %s ..." % file)
        r = requests.post("http://%s:%d/upload" %(OTA_SERVER, OTA_WEBSERVER_PORT), data=data, files=files).json()

    except Exception as e:
        error(e)

    if r['res']['headers']['etag'] != ("\"%s\"" % md5):
        error("verify error, remote md5: %s" % r['res']['headers']['etag'])

    click.echo("upload success, verify ok")

    return (r['url'], size, md5)

# The callback when connect to mqtt broker
def on_connect(client, (dn, pk, payload), flags, rc):
    global bar
    # print userdata
    topic_upgrade = "/%s/%s/upgrade" % (pk, dn)
    topic_progress = "/%s/%s/progress" % (pk, dn)

    click.echo("connected mqtt broker, waiting to finish erasing flash...")

    client.publish(topic_upgrade, payload, qos=1)
    client.subscribe(topic_progress, qos=1)

# The callback for when a PUBLISH message is received from the mqtt broker.
def on_message(client, (dn, pk, payload), msg):
    global first_progress, bar, timer

    #print "on message"
    timer.cancel()
    timer = threading.Timer(5, mqtt_timeout, [client])
    timer.damon = True
    timer.start()

    # print userdata
    # click.echo("%s %s" % (msg.topic, str(msg.payload)))
    payload = json.loads(str(msg.payload))

    if bar == None:
        widgets=[
            progressbar.Percentage(),
            progressbar.Bar(),
            ' (', progressbar.ETA(), ') ',
        ]
        bar = progressbar.ProgressBar(max_value=100, widgets=widgets, redirect_stdout=True)
        bar.start()
    bar.update(payload["progress"])
    if payload["progress"] == 100:
        bar.finish()
        click.secho("Congratulations!! upgrade done", fg="green")
        timer.cancel()
        sys.exit(0)

# start mqtt client
def mqtt_operation(dn, pk, payload):

    global timer
    client = mqtt.Client(userdata=(dn, pk, payload))
    client.on_connect = on_connect
    client.on_message = on_message
    token = base64.b64decode(OTA_EMQ_TOKEN).split('|')
    if len(token) != 2:
        error("wrong EMQ token")
    client.username_pw_set(username=token[0], password=token[1])

    client.connect(OTA_SERVER, OTA_EMQ_PORT, 60)
    timer = threading.Timer(15, mqtt_timeout, [client])
    timer.damon = True
    timer.start()
    client.loop_forever()

# exit when timeout
def mqtt_timeout(client):
    global timer
    timer.cancel()
    client.disconnect()
    error("mqtt timeout")



import sys

__version__ = "0.3.4"
__title__ = "aos-cube"
__description__ = (
    "aos commmand line tool for AliOS-Things development.")
__url__ = ""

__author__ = "Alibaba"
__email__ = "aliosthings@service.aliyun.com"

__license__ = "Apache Software License"
__copyright__ = "Copyright 2016-present aos-cube"

if sys.version_info < (2, 7, 0) or sys.version_info >= (3, 0, 0):
    msg = ("aos cube is compatible with Python version >= 2.7 and < 3.0\n")
    sys.stderr.write(msg % (__version__, sys.version))
    sys.exit(1)



import os, sys
import subprocess
import shutil
import traceback

import click
from aos.util import log, error, cd_aos_root, is_domestic, popen, get_board_dir
from aos.program import *

# Make command
@click.command("make", short_help="Make aos program/component")
@click.argument("targets", required=False, nargs=-1, metavar="[TARGETS...]")
@click.option("-c", "--cmd", metavar="[CMD]", help="Sub build stage for target")
def cli(targets, cmd):
    """ Make aos program/component """
    from aos import __version__
    log('aos-cube version: %s\n' % __version__)

    make_args = ' '.join(targets)
    for arg in targets:
        if '@' in arg:
            targets = arg.split('@')
            if len(targets) < 2:
                error('Must special app and board when build aos')

            board = targets[1]
            if cmd:
                make_args = re.sub(arg, "%s.%s" % (arg, cmd), make_args)

            if board in get_scons_enabled_boards():
                scons_build(targets)
            else:
                make_build(make_args)

            return
    #aos make clean go here
    make_build(make_args)

#
# Common functions
#
def download_toolchains(downloads):
    tmp_dir = 'tmp_{0:02x}'.format(ord(os.urandom(1)))
    while os.path.isdir(tmp_dir):
        tmp_dir = 'tmp_{0:02x}'.format(ord(os.urandom(1)))
    try:
        os.mkdir(tmp_dir)
    except:
        print('download toolchains failed: unable to create a temp folder')
        return
    try:
        os.chdir(tmp_dir)
        for download in downloads:
            result = 0
            name, git_url, dst_dir = download
            print('toolchain {} missing, start download ...'.format(name))
            print(git_url + ' -> ' + dst_dir)
            result += subprocess.call(['git', 'clone', '--depth=1', git_url, name])
            if result > 0:
                print('git clone toolchain {} failed'.format(name))
                print('You can mannually try fix this problem by running:')
                print('    git clone {} {}'.format(git_url, name))
                print('    mv {0}/main {1} && rm -rf {0}'.format(name, dst_dir))
            src_dir = name + '/main'
            if os.path.exists(src_dir) == False:
                print('toolchain folder {} none exist'.format(src_dir))
                result += 1
            if result == 0:
                dst_dir = '../' + dst_dir
                if os.path.isfile(dst_dir) == True:
                    os.remove(dst_dir)
                if os.path.isdir(dst_dir) == True:
                    shutil.rmtree(dst_dir)
                if os.path.isdir(os.path.dirname(dst_dir)) == False:
                    os.makedirs(os.path.dirname(dst_dir))
                shutil.move(src_dir, dst_dir)
                print('download toolchain {} succeed'.format(name))
            else:
                print('download toolchain {} failed'.format(name))
    except:
        traceback.print_exc()
    finally:
        os.chdir('../')
    try:
        shutil.rmtree(tmp_dir)
    except:
        print("toolchain auto-install error: can not remove temp folder {}, please remove it manually".format(tmp_dir))
        pass

def is_specific_path(toolchain):
    try:
        if toolchain['path_specific']:
            return True
    except KeyError as exc:
        return False
    return False

def _install_toolchains(build_args):
    board = None
    host_os = get_host_os()
    if host_os == None:
        error('Unsupported Operating System!')

    #cd to aos root_dir
    ret, original_dir = cd_aos_root()
    if ret != 'success':
        error("not in AliOS-Things source code directory")

    #check config file to be enable this function (for backward compatability)
    autodownload_enable = False
    if os.path.exists('build/toolchain_config.py'):
        sys.path.append("./build")
        try:
            from toolchain_config import auto_download
        except:
            error("Import toolchain configs failed")

        if auto_download == "yes":
            autodownload_enable = True
    elif os.path.exists('build/toolchain_autodownload.config'):
        try:
            with open('build/toolchain_autodownload.config') as file:
                if 'yes' in file.read(): autodownload_enable = True
        except:
            pass

    if autodownload_enable == False:
        if os.path.isdir(original_dir): os.chdir(original_dir)
        return

    for arg in build_args:
        if '@' not in arg:
            continue
        args = arg.split('@')
        board = args[1]
        break

    if not board:
        board = get_config_value("AOS_BUILD_BOARD")

    if not board:
        if os.path.isdir(original_dir):
            os.chdir(original_dir)
        return

    if not os.path.isdir(os.path.join('board', board)):
        board_dir = get_board_dir(board)
        if not board_dir:
            error('Can not find board {}'.format(board))

    downloads = []
    if os.path.exists('build/toolchain_config.py'):
        from toolchain_config import boards
    else:
        from aos.constant import boards

    if board in boards:
        print('Check if required tools for {} exist'.format(board))
        for toolchain in boards[board]:
            name = toolchain['name']
            command = toolchain['command']
            version = toolchain['version']
            if is_specific_path(toolchain) is True:
                cmd_path = '{}/bin/{}'.format(toolchain['path'], command)
            else:
                cmd_path = '{}/{}/bin/{}'.format(toolchain['path'], host_os, command)
            if cmd_version_match(cmd_path, version) == True:
                continue
            if toolchain['use_global'] and cmd_version_match(command, version) == True:
                continue
            git_url = toolchain['{}_url'.format(host_os)]
            if git_url == '':
                continue
            if is_domestic() is False:
                git_url = git_url.replace('gitee', 'github')
                git_url = git_url.replace('alios-things', 'aliosthings')

            if is_specific_path(toolchain) is True:
                downloads.append([name, git_url, '{}'.format(toolchain['path'])])
            else:
                downloads.append([name, git_url, '{}/{}'.format(toolchain['path'], host_os)])
    if len(downloads): download_toolchains(downloads)

    if os.path.isdir(original_dir): os.chdir(original_dir)

#
# Support for scons build
#
def scons_build(args):
    if os.path.exists('ucube.py') == True:
        make_args = ['scons -j4 -f ucube.py']
    else:
        make_args = ['scons -j4 -f build/ucube.py']

    target_find = False

    for arg in args:
        if '@' in arg and not target_find:
            targets = arg.split('@')
            if len(targets) < 2:
                error('Must special app and board when build aos')

            app = 'APPLICATION='+targets[0]
            board = 'BOARD=' + targets[1]
            make_args.append(app)
            make_args.append(board)

            if len(targets) == 3:
                build_type = 'TYPE=' + targets[2]
                make_args.append(build_type)

            target_find = True
        elif arg.startswith('JOBS=') and arg.replace('JOBS=', '').isdigit() == True:
            jnum = arg.replace('JOBS=', '-j')
            make_args[0] = make_args[0].replace('-j4', jnum)
        else:
            make_args.append(arg)

    popen(' '.join(make_args), shell=True, cwd=os.getcwd())

def get_scons_enabled_boards():
    if os.path.exists("build/scons_enabled.py"):
        module_path = os.path.abspath(os.path.join('build'))
        sys.path.append(module_path)
        from scons_enabled import scons_enabled_boards
        return scons_enabled_boards
    else:
        return []

#
# Support for make build
#
def make_build(make_args):
    # aos new program
    if os.path.isfile(os.path.join(os.getcwd(), Cfg.file)):
        pd = Program(os.getcwd())
        os_path = pd.get_cfg(OS_PATH)

        build_dir = ''
        out_path = ''
        is_build_dir = 'BUILD_DIR=' in make_args or os.environ.get('BUILD_DIR')
        if not is_build_dir:
            out_path = os.path.join(os.getcwd(), 'out').replace(os.path.sep, '/')
            build_dir = 'BUILD_DIR=' + out_path

        # cube_makefile = 'CUBE_MAKEFILE=' + os.path.join(pd.get_cfg(PROGRAM_PATH), CUBE_MAKEFILE).replace(os.path.sep, '/')
        app_dir = 'APPDIR=' + os.path.dirname(os.getcwd())

        with cd(os_path):
            cube_modify = pd.get_cfg(CUBE_MODIFY)
            if cube_modify == '1':
                _run_make(['-e', '-f build/Makefile', 'clean'])
                if build_dir:
                    shutil.rmtree(out_path, True)
                pd.set_cfg(CUBE_MODIFY, '0')
            _run_make(['-e', '-f build/Makefile', make_args, app_dir, build_dir])
    else:
        # aos source code
        _run_make(['-e', '-f build/Makefile', make_args])

def _run_make(arg_list):
    #install dependent toolchains
    download_toolchain = "yes"
    target_no_toolchain = [".config", ".menuconfig", "clean", "distclean", "help", "export-keil", "export-iar", "_defconfig"]
    for arg in arg_list:
        for target in target_no_toolchain:
            if target in arg:
                download_toolchain = "no"
                break

    if download_toolchain == "yes":
        _install_toolchains(sys.argv[2:])

    # check operating system
    host_os = get_host_os()
    if host_os == None:
        error('Unsupported Operating System!')

    #cd to aos root_dir
    ret, original_dir = cd_aos_root()
    if ret != 'success':
        error("not in AliOS-Things source code directory")

    make_cmds = {
        'Win32': 'cmd/win32/make.exe',
        'Linux64': 'cmd/linux64/make',
        'Linux32': 'cmd/linux32/make',
        'OSX': 'cmd/osx/make'
        }
    tools_dir = os.path.join(os.getcwd(), 'build').replace('\\', '/')
    make_cmd = os.path.join(tools_dir, make_cmds[host_os])

    # Run make command
    make_cmd_str = ' '.join([make_cmd, 'HOST_OS=' + host_os, 'TOOLS_ROOT=' + tools_dir] + list(arg_list))
    popen(make_cmd_str, shell=True, cwd=os.getcwd())
    if os.path.isdir(original_dir): os.chdir(original_dir)



import os
import re
import click

from aos.constant import *
from aos.managers.comp import CompManager
from aos.util import update_config_in
import ConfigParser

def validate_boards(ctx, param, value):
    return value

@click.command("init", short_help="Initialize AOS project")
@click.option("-d", "--project-dir", default=os.getcwd, help="Directory for initializing AOS project",
    type=click.Path(file_okay=False, dir_okay=True, writable=True, resolve_path=True))
@click.option("-b", "--board", multiple=True, metavar="BOARD", callback=validate_boards, help="Board component to install")
@click.option("-c", "--component", multiple=True, metavar="COMPONENT_LIST", help="Other component to inistall (eg. App, Middleware ...)")
@click.option("-a", "--all", is_flag=True, help="Install all available components")
@click.option("-s", "--silent", is_flag=True, help="Suppress progress reporting")
@click.option("-t", "--check-available", is_flag=True, help="Check if component available before installation")
@click.option("-u", "--baseurl", metavar="URL", help="Base URL of AliOS Things release repo")
@click.pass_context
def cli(ctx, project_dir, board, component, all, silent, check_available, baseurl):
    if not silent:
        click.echo("\nInitialize AliOS Things Project in %s" % \
            click.style(os.path.join(project_dir, OS_NAME), fg="cyan"))

    init_project_dir(project_dir)
    yumconf = os.path.join(project_dir, ".aos", "yum.conf")
    if os.path.isfile(yumconf):
        os.remove(yumconf)

    complist = []
    cm = CompManager(baseurl)
    cm.clean()
    if all:
        remote = True
        opts = []
        output, err = cm.list(remote, *opts)
        patten = re.compile(r"(.*)\.noarch\s*(.*\.aos)\s*")
        for line in output.splitlines():
            match = patten.match(line)
            if match:
                compname = "%s-%s" % (match.group(1), match.group(2))
                complist += [compname]

    if board:
        complist += board
    if component:
        complist += component

    if complist:
        if "board_" in " ".join(complist):
            cm.install(complist + OS_DEF_COMPS, check_available)
        else:
            cm.install(complist, check_available)

    os.chdir(os.path.join(project_dir, OS_NAME))
    update_config_in()

def init_project_dir(project_dir):
    if not os.path.isdir(project_dir):
        os.makedirs(project_dir)

    dot_aos = os.path.join(project_dir, ".aos")
    if not os.path.isdir(dot_aos):
        os.makedirs(dot_aos)

    source_dir = os.path.join(project_dir, OS_NAME)
    if not os.path.isdir(source_dir):
        os.makedirs(source_dir)

    download_dir = os.path.join(dot_aos, "downloads")
    if not os.path.isdir(download_dir):
        os.makedirs(download_dir)

    cf = ConfigParser.ConfigParser()
    cf.add_section("global")
    cf.set("global", "project_dir", project_dir)

    config_file = os.path.join(dot_aos, OS_CONFIG)
    cf.write(open(config_file, "w"))



import os, sys
from setuptools import find_packages, setup
sys.path.append('./aos')
from aos import __title__, __version__, __description__, __url__, __author__, __email__

LONG_DESC = open('pypi_readme.rst').read()
LICENSE = open('LICENSE').read()

install_requires = [
    'pyserial',
    'esptool',
    'scons',
    'requests',
    'click',
    'paho-mqtt',
    'progressbar2'
]

setup(
    name=__title__,
    version=__version__,
    description=__description__,
    long_description=LONG_DESC,
    url=__url__,
    author=__author__,
    author_email=__email__,
    license=LICENSE,
    python_requires='>=2.7, <3',
#    packages=["aos", "aos.commands", "aos.managers", "udc"],
    packages=find_packages(),
    package_dir={'aos':'aos', 'udc':'udc'},
    package_data={'aos': ['.vscode/*'], 'udc': ['controller_certificate.pem', 'board/*/*']},
    install_requires=install_requires,
    entry_points={
        'console_scripts': [
            'aos=aos.__main__:main',
            'udc=udc.udc:main',
            'aos-cube=aos.__main__:main',
        ]
    },
)



import os, sys, time, platform, json, traceback, random, re, glob, uuid
import socket, ssl, thread, threading, subprocess, signal, Queue, importlib
import select
from os import path
import packet as pkt

MAX_MSG_LENGTH = 65536
ENCRYPT = True
DEBUG = True
EN_STATUS_POLL = True
LOCALLOG = False

def signal_handler(sig, frame):
    print "received SIGINT"
    raise KeyboardInterrupt

def queue_safeput(queue, item):
    try:
        queue.put(item, False)
    except:
        pass

class ConnectionLost(Exception):
    pass

class Client:
    def __init__(self):
        self.service_socket = None
        self.output_queue = Queue.Queue(256)
        self.debug_queue = Queue.Queue(256)
        self.devices = {}
        self.keep_running = True
        self.connected = False
        self.poll_str = '\x1b[t'
        bytes = os.urandom(4)
        for byte in bytes:
            self.poll_str += '{0:02x}'.format(ord(byte))
        self.poll_str += 'm'
        self.poll_interval = 60
        self.uuid = None
        self.model_interface = {}
        self.mesh_changed = [re.compile('become leader'),
                             re.compile('become detached'),
                             re.compile('allocate sid 0x[0-9a-f]{4}, become [0-9] in net [0-9a-f]{4}')]
        self.neighbor_changed = [re.compile('sid [0-9a-f]{4} mac [0-9a-f]{16} is replaced'),
                                 re.compile('[0-9a-f]{1,4} neighbor [0-9a-f]{16} become inactive')]
        self.device_uuid_changed = ["ACCS: connected",
                             "ACCS: disconnected",
                             'GATEWAY: connect to server succeed']

    def packet_send_thread(self):
        heartbeat_timeout = time.time() + 10
        while self.keep_running:
            try:
                [type, content] = self.output_queue.get(block=True, timeout=0.1)
            except Queue.Empty:
                type = None
                pass
            if self.service_socket == None:
                continue
            if type == None:
                if time.time() < heartbeat_timeout:
                    continue
                heartbeat_timeout += 10
                data = pkt.construct(pkt.HEARTBEAT,'')
            else:
                data = pkt.construct(type, content)
            try:
                self.service_socket.send(data)
            except:
                self.connected = False
                continue

    def send_packet(self, type, content, timeout=0.1):
        if self.service_socket == None:
            return False
        try:
            self.output_queue.put([type, content], True, timeout)
            return True
        except Queue.Full:
            print "error: ouput buffer full, drop packet [{0] {1}]".format(type, content)
        return False

    def debug_daemon_thread(self):
        debug_sessions = {}
        while self.keep_running:
            if debug_sessions == {} or self.debug_queue.empty() == False:
                [type, term, device] = self.debug_queue.get()
                if device not in self.devices:
                    content = term + ',' + device + ':' + 'nonexist'
                    self.send_packet(type, content)
                    continue
                interface = self.devices[device]['interface']
                if type == pkt.DEVICE_DEBUG_START:
                    if 'debug_socket' in self.devices[device]: #session exist
                        sock = self.devices[device]['debug_socket']
                        if debug_sessions[sock]['term'] != term:
                            content = term + ',' + device + ':' + 'busy'
                            self.send_packet(type, content)
                            continue
                        interface.debug_stop(device) #stop current session
                        debug_sessions.pop(sock)
                        sock.close()

                    if hasattr(interface, 'debug_start') == False or hasattr(interface, 'debug_stop') == False:
                        content = term + ',' + device + ':' + 'debug unsupported'
                        self.send_packet(type, content)
                        continue

                    used_ports = []
                    for session in debug_sessions:
                        used_ports.append(debug_sessions[session]['port'])
                    port = 3096 + ord(os.urandom(1)) * ord(os.urandom(1)) / 64
                    while port in used_ports:
                        port = 3096 + ord(os.urandom(1)) * ord(os.urandom(1)) / 64

                    result = interface.debug_start(device, port)
                    if result != 'success':
                        print "start st-util for {0} at port {1} failed, ret={2}".format(device, port, result)
                        content = term + ',' + device + ':' + result
                        self.send_packet(type, content)
                        continue

                    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    try:
                        sock.connect(('127.0.0.1', port))
                        sock.setblocking(0)
                    except:
                        sock = None
                    if sock:
                        debug_sessions[sock] = {'term': term, 'device': device, 'port': port}
                        self.devices[device]['debug_socket'] = sock
                        content = term + ',' + device + ':' + 'success'
                        print 'start debugging device {0} at port {1} succeed'.format(device, port)
                    else:
                        if DEBUG: traceback.print_exc()
                        interface.debug_stop(device)
                        content = term + ',' + device + ':' + 'fail'
                        print 'start debugging device {0} failed'.format(device)
                    self.send_packet(type, content)
                    continue
                elif type in [pkt.DEVICE_DEBUG_REINIT, pkt.DEVICE_DEBUG_STOP]:
                    operations = {pkt.DEVICE_DEBUG_REINIT: 'reinit', pkt.DEVICE_DEBUG_STOP: 'stop'}
                    sock = None
                    for session in debug_sessions:
                        if debug_sessions[session]['term'] != term:
                            continue
                        if debug_sessions[session]['device'] != device:
                            continue
                        sock = session
                        break
                    if sock == None:
                        print 'error: try to {0} a nonexisting debug session'.format(operations[type])
                        content = term + ',' + device + ':' + 'session nonexist'
                        self.send_packet(type, content)
                        continue

                    port = debug_sessions[sock]['port']
                    interface.debug_stop(device)
                    sock.close()
                    self.devices[device].pop('debug_socket')
                    debug_sessions.pop(sock)
                    if type == pkt.DEVICE_DEBUG_STOP:
                        content = term + ',' + device + ':' + 'success'
                        self.send_packet(type, content)
                        print 'stop debugging {0} succeed'.format(device)
                    elif type == pkt.DEVICE_DEBUG_REINIT:
                        result = interface.debug_start(device, port)
                        if result != 'success':
                            content = term + ',' + device + ':' + 'fail'
                            self.send_packet(type, content)
                            continue
                        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                        try:
                            sock.connect(('127.0.0.1', port))
                            sock.setblocking(0)
                        except:
                            if DEBUG: traceback.print_exc()
                            sock = None
                        if sock:
                            debug_sessions[sock] = {'term': term, 'device': device, 'port': port}
                            self.devices[device]['debug_socket'] = sock
                            content = term + ',' + device + ':' + 'success'
                            self.send_packet(type, content)
                            print 'restart debugging {0} succeed'.format(device)
                        else:
                            interface.debug_stop(device)
                            self.devices[device].pop('debug_socket')
                            content = term + ',' + device + ':' + 'fail'
                            self.send_packet(type, content)
                            print 'restart debugging {0} failed, debug session closed'.format(device)
                    continue
                else:
                    print "error: wrong command type '{0}'".format(type)
                continue

            r, w, e = select.select(list(debug_sessions), [], [], 0.02)
            for sock in r:
                term = debug_sessions[sock]['term']
                device = debug_sessions[sock]['device']
                try:
                    data = sock.recv(MAX_MSG_LENGTH)
                except:
                    data = None
                if not data:
                    print 'debug session for {0}:{1} closed'.format(term, device)
                    interface = self.devices[device]['interface']
                    interface.debug_stop(device)
                    debug_sessions.pop(sock)
                    self.devices[device].pop('debug_socket')
                    content = term + ',' + device + ':' + 'session closed'
                    self.send_packet(pkt.DEVICE_DEBUG_STOP, content)
                    continue
                content = term + ',' + device + ':' + data
                self.send_packet(pkt.DEVICE_DEBUG_DATA, content)

    def send_device_list(self):
        device_list = []
        for device in list(self.devices):
            if self.devices[device]['valid']:
                device_list.append(device)
        content = ':'.join(device_list)
        self.send_packet(pkt.CLIENT_DEV, content)

    def send_device_status(self):
        for device in list(self.devices):
            if self.devices[device]['valid'] == False:
                continue
            content = device + ':' + json.dumps(self.devices[device]['attributes'], sort_keys=True)
            ret = self.send_packet(pkt.DEVICE_STATUS, content)
            if ret == False:
                break

    def run_poll_command(self, device, command, lines_expect, timeout):
        filter = {}
        response = []
        while self.devices[device]['plog_queue'].empty() == False:
            self.devices[device]['plog_queue'].get()
        self.devices[device]['handle'].write(self.poll_str + command + '\r')
        start = time.time()
        while True:
            try:
                log = self.devices[device]['plog_queue'].get(False)
            except:
                log = None
            if time.time() - start >= timeout:
                break
            if log == None:
                time.sleep(0.01)
                continue
            log = log.replace('\r', '')
            log = log.replace('\n', '')
            log = log.replace(self.poll_str, '')
            if log == '':
                continue
            response.append(log)
            if len(response) > lines_expect:
                break

        if len(response) > 0:
            response.pop(0)
        if not response:
            print "device {0} run poll commad '{1}' faild".format(device, command)
        return response

    def device_cmd_process(self, device, exit_condition):
        poll_fail_num = 0
        interface = self.devices[device]['interface']
        pcmd_queue = self.devices[device]['pcmd_queue']
        if self.devices[device]['attributes'] != {}:
            content = device + ':' + json.dumps(self.devices[device]['attributes'], sort_keys=True)
            self.send_packet(pkt.DEVICE_STATUS, content)
        poll_timeout = time.time() + 3 + random.uniform(0, self.poll_interval/10)
        while interface.exist(device) and exit_condition.is_set() == False:
            try:
                if EN_STATUS_POLL == True and time.time() >= poll_timeout:
                    poll_timeout += self.poll_interval
                    queue_safeput(pcmd_queue, ['devname', 1, 0.2])
                    queue_safeput(pcmd_queue, ['mac', 1, 0.2])
                    queue_safeput(pcmd_queue, ['version', 2, 0.2])
                    queue_safeput(pcmd_queue, ['uuid', 1, 0.2])
                    queue_safeput(pcmd_queue, ['umesh status', 11, 0.2])
                    queue_safeput(pcmd_queue, ['umesh extnetid', 1, 0.2])
                    queue_safeput(pcmd_queue, ['umesh nbrs', 35, 0.3])

                block=True
                timeout=0
                try:
                    args = None
                    if self.devices[device]['ucmd_queue'].empty() == True and pcmd_queue.empty() == True:
                        args = self.devices[device]['ucmd_queue'].get(block=True, timeout=0.1)
                    elif self.devices[device]['ucmd_queue'].empty() == False:
                        args = self.devices[device]['ucmd_queue'].get()
                except Queue.Empty:
                    args = None
                    continue
                except:
                    if DEBUG: traceback.print_exc()
                    args = None
                    continue

                if args != None:
                    type = args[0]
                    term = args[1]
                    if type == pkt.DEVICE_ERASE:
                        self.device_erase(device, term)
                    elif type == pkt.DEVICE_PROGRAM:
                        address = args[2]
                        filename = args[3]
                        self.device_program(device, address, filename, term)
                    elif type in [pkt.DEVICE_RESET, pkt.DEVICE_START, pkt.DEVICE_STOP]:
                        self.device_control(device, type, term)
                    elif type == pkt.DEVICE_CMD:
                        cmd = args[2]
                        self.device_run_cmd(device, cmd, term)
                        if re.search('umesh extnetid [0-9A-Fa-f]{12}', cmd) != None:
                            queue_safeput(pcmd_queue, ['umesh extnetid', 1, 0.2])
                    else:
                        print "error: unknown operation type {0}".format(repr(type))
                    args = None
                    time.sleep(0.05)
                    continue

                if pcmd_queue.empty() == True:
                    continue

                [cmd, lines, timeout] = pcmd_queue.get()
                response = self.run_poll_command(device, cmd, lines, timeout)

                if cmd == 'devname': #poll device model
                    if len(response) == lines and response[0].startswith('device name:'):
                        poll_fail_num = 0
                        self.devices[device]['attributes']['model'] = response[0].split()[-1]
                    else:
                        poll_fail_num += 1
                elif cmd == 'mac': #poll device mac
                    if len(response) == 1 and response[0].startswith('MAC address:'):
                        poll_fail_num = 0
                        macaddr = response[0].split()[-1]
                        macaddr = macaddr.replace('-', '') + '0000'
                        self.devices[device]['attributes']['macaddr'] = macaddr
                    else:
                        poll_fail_num += 1
                elif cmd == 'version': #poll device version
                    if len(response) == lines:
                        poll_fail_num = 0
                        for line in response:
                            if 'kernel version :' in line:
                                self.devices[device]['attributes']['kernel_version'] = line.replace('kernel version :AOS-', '')
                            if 'app version :' in line:
                                line = line.replace('app version :', '')
                                line = line.replace('app-', '')
                                line = line.replace('APP-', '')
                                self.devices[device]['attributes']['app_version'] = line
                    else:
                        poll_fail_num += 1
                elif cmd == 'umesh status': #poll mesh status
                    if len(response) == lines:
                        poll_fail_num = 0
                        for line in response:
                            if 'state\t' in line:
                                self.devices[device]['attributes']['state'] = line.replace('state\t', '')
                            elif '\tnetid\t' in line:
                                self.devices[device]['attributes']['netid'] = line.replace('\tnetid\t', '')
                            elif '\tsid\t' in line:
                                self.devices[device]['attributes']['sid'] = line.replace('\tsid\t', '')
                            elif '\tnetsize\t' in line:
                                self.devices[device]['attributes']['netsize'] = line.replace('\tnetsize\t', '')
                            elif '\trouter\t' in line:
                                self.devices[device]['attributes']['router'] = line.replace('\trouter\t', '')
                            elif '\tchannel\t' in line:
                                self.devices[device]['attributes']['channel'] = line.replace('\tchannel\t', '')
                    else:
                        poll_fail_num += 1
                elif cmd == 'umesh nbrs': #poll mesh nbrs
                    if len(response) > 0 and 'num=' in response[-1]:
                        poll_fail_num = 0
                        nbrs = {}
                        for line in response:
                            if '\t' not in line or ',' not in line:
                                continue
                            line = line.replace('\t', '')
                            nbr_info = line.split(',')
                            if len(nbr_info) < 10:
                                continue
                            nbrs[nbr_info[0]] = {'relation':nbr_info[1], \
                                                 'netid':nbr_info[2], \
                                                 'sid':nbr_info[3], \
                                                 'link_cost':nbr_info[4], \
                                                 'child_num':nbr_info[5], \
                                                 'channel':nbr_info[6], \
                                                 'reverse_rssi':nbr_info[7], \
                                                 'forward_rssi':nbr_info[8], \
                                                 'last_heard':nbr_info[9]}
                            if len(nbr_info) > 10:
                                nbrs[nbr_info[0]]['awake'] = nbr_info[10]
                        self.devices[device]['attributes']['nbrs'] = nbrs
                    else:
                        poll_fail_num += 1
                elif cmd == 'umesh extnetid': #poll mesh extnetid
                    if len(response) == 1 and response[0].count(':') == 5:
                        poll_fail_num += 1
                        self.devices[device]['attributes']['extnetid'] = response[0]
                    else:
                        poll_fail_num += 1
                elif cmd == 'uuid': #poll uuid
                    if len(response) == 1:
                        if 'uuid:' in response[0]:
                            poll_fail_num = 0
                            self.devices[device]['attributes']['uuid'] = response[0].replace('uuid: ', '')
                        elif 'alink is not connected' in response[0]:
                            poll_fail_num = 0
                            self.devices[device]['attributes']['uuid'] = 'N/A'
                        else:
                            poll_fail_num += 1
                    else:
                        poll_fail_num += 1
                else:
                    print "error: unrecognized poll cmd '{0}'".format(cmd)
                    continue

                if poll_fail_num >= 7:
                    if self.devices[device]['attributes']['status'] == 'active':
                        print "device {0} become inactive".format(device)
                    self.devices[device]['attributes']['status'] = 'inactive'
                else:
                    if self.devices[device]['attributes']['status'] == 'inactive':
                        print "device {0} become active".format(device)
                    self.devices[device]['attributes']['status'] = 'active'

                if pcmd_queue.empty() == False:
                    continue
                content = device + ':' + json.dumps(self.devices[device]['attributes'], sort_keys=True)
                self.send_packet(pkt.DEVICE_STATUS, content)
            except:
                if interface.exist(device) == False:
                    exit_condition.set()
                    break
                if exit_condition.is_set() == True:
                    break
                if DEBUG: traceback.print_exc()
                self.devices[device]['handle'].close()
                self.devices[device]['handle'].open()
        print 'devie command process thread for {0} exited'.format(device)

    def device_log_filter(self, device, log):
        pcmd_queue = self.devices[device]['pcmd_queue']
        if EN_STATUS_POLL == False:
            return
        if pcmd_queue.full() == True:
            return
        for flog in self.mesh_changed:
            if flog.search(log) == None:
                continue
            #print log
            #print "device {0} mesh status changed".format(device)
            queue_safeput(pcmd_queue, ['umesh status', 11, 0.2])
            queue_safeput(pcmd_queue, ['umesh nbrs', 33, 0.3])
            return
        for flog in self.neighbor_changed:
            if flog.search(log) == None:
                continue
            #print log
            #print "device {0} neighbors changed".format(device)
            queue_safeput(pcmd_queue, ['umesh nbrs', 33, 0.3])
            return
        for flog in self.device_uuid_changed:
            if flog not in log:
                continue
            #print log
            #print "device {0} uuid changed".format(device)
            queue_safeput(pcmd_queue, ['uuid', 1, 0.2])
            return

    def device_log_poll(self, device, exit_condition):
        log_time = time.time()
        log = ''
        if LOCALLOG:
            logfile= path.join(path.expanduser('~'), '.udclient', path.basename(device) + '.log')
            flog = open(logfile, 'a+')
        interface = self.devices[device]['interface']
        while interface.exist(device) and exit_condition.is_set() == False:
            if self.connected == False or self.devices[device]['iolock'].locked():
                time.sleep(0.01)
                continue

            newline = False
            while self.devices[device]['iolock'].acquire(False) == True:
                try:
                    c = self.devices[device]['handle'].read(1)
                except:
                    c = ''
                finally:
                    self.devices[device]['iolock'].release()

                if c == '':
                    break

                if log == '':
                    log_time = time.time()
                log += c
                if c == '\n':
                    newline = True
                    break

            if newline == True and log != '':
                if self.poll_str in log:
                    queue_safeput(self.devices[device]['plog_queue'], log)
                else:
                    self.device_log_filter(device, log)
                if LOCALLOG:
                    flog.write('{0:.3f}:'.format(log_time) + log)
                log = device + ':{0:.3f}:'.format(log_time) + log
                self.send_packet(pkt.DEVICE_LOG,log)
                log = ''
        if LOCALLOG:
            flog.close()
        print 'device {0} removed'.format(device)
        self.devices[device]['valid'] = False
        exit_condition.set()
        self.devices[device]['handle'].close()
        self.send_device_list()
        print 'device log poll thread for {0} exited'.format(device)

    def add_new_device(self, mi, device):
        handle = mi.new_device(device)
        if handle == None:
            return False
        self.devices[device] = {
            'valid':True, \
            'handle':handle, \
            'interface' : mi, \
            'iolock':threading.Lock(), \
            'attributes':{}, \
            'ucmd_queue':Queue.Queue(12), \
            'pcmd_queue':Queue.Queue(64), \
            'plog_queue':Queue.Queue(64)
            }
        self.devices[device]['attributes']['status'] = 'inactive'
        return True

    def add_old_device(self, mi, device):
        ser = mi.new_device(device)
        if ser == None:
            return False
        self.devices[device]['handle'] = ser
        if self.devices[device]['iolock'].locked():
            self.devices[device]['iolock'].release()
        while self.devices[device]['ucmd_queue'].empty() == False:
            self.devices[device]['ucmd_queue'].get()
        while self.devices[device]['pcmd_queue'].empty() == False:
            self.devices[device]['pcmd_queue'].get()
        while self.devices[device]['plog_queue'].empty() == False:
            self.devices[device]['plog_queue'].get()
        self.devices[device]['attributes']['status'] = 'inactive'
        self.devices[device]['valid'] = True
        return True

    def list_devices(self):
        os = platform.system()
        devices_new = []
        for model in self.model_interface:
            mi = self.model_interface[model]

            devices = mi.list_devices(os)
            for device in devices:
                if device in self.devices and self.devices[device]['valid'] == True:
                    continue
                if device not in self.devices:
                    ret = self.add_new_device(mi, device)
                else:
                    ret = self.add_old_device(mi, device)
                if ret == True:
                    devices_new.append(device)

        devices_new.sort()
        return devices_new

    def device_monitor(self):
        while self.keep_running:
            devices_new = self.list_devices()
            for device in devices_new:
                print 'device {0} added'.format(device)
                exit_condition = threading.Event()
                thread.start_new_thread(self.device_log_poll, (device, exit_condition,))
                thread.start_new_thread(self.device_cmd_process, (device, exit_condition,))
            if devices_new != []:
                self.send_device_list()
            time.sleep(0.5)
        print 'device monitor thread exited'
        self.keep_running = False

    def load_interfaces(self):
        board_dir = path.join(path.dirname(path.abspath(__file__)), 'board')
        candidates = os.listdir(board_dir)
        for d in candidates:
            if path.isdir(path.join(board_dir, d)) == False:
                continue
            model = path.basename(d)
            interface_file = path.join(board_dir, d, model+'.py')
            if path.isfile(interface_file) == False:
                continue
            sys.path.append(path.join(board_dir, d))
            try:
                self.model_interface[model] =  importlib.import_module(model)
            except:
                if DEBUG: traceback.print_exc()
                continue
            print 'model loaded - {0}'.format(model)

    def device_erase(self, device, term):
        interface = self.devices[device]['interface']
        self.devices[device]['iolock'].acquire()
        try:
            ret = interface.erase(device)
        except:
            if DEBUG: traceback.print_exc()
            ret = 'fail'
        finally:
            self.devices[device]['iolock'].release()
        print 'erasing', device, '...', ret
        content = term + ',' + ret
        self.send_packet(pkt.DEVICE_ERASE, content)

    def device_program(self, device, address, file, term):
        if device not in self.devices:
            print "error: progamming nonexist device {0}".format(device)
            content = term + ',' + 'device nonexist'
            self.send_packet(pkt.DEVICE_PROGRAM, content)
            return

        interface = self.devices[device]['interface']
        self.devices[device]['iolock'].acquire()
        try:
            ret = interface.program(device, address, file)
        except:
            if DEBUG: traceback.print_exc()
            ret = 'fail'
        finally:
            self.devices[device]['iolock'].release()
        print 'programming', file, 'to', device, '@', address, '...', ret
        content = term + ',' + ret
        self.send_packet(pkt.DEVICE_PROGRAM, content)

    def device_control(self, device, type, term):
        operations= {pkt.DEVICE_RESET:'reset', pkt.DEVICE_STOP:'stop', pkt.DEVICE_START:'start'}
        if device not in self.devices:
            print "error: controlling nonexist device {0}".format(device)
            content = term + ',' + 'device nonexist'
            self.send_packet(type, content)
            return

        interface = self.devices[device]['interface']
        try:
            ret = interface.control(device, operations[type])
        except:
            if DEBUG: traceback.print_exc()
            ret = 'fail'
        print operations[type], device, ret
        content = term + ',' + ret
        self.send_packet(type, content)

    def device_run_cmd(self, device, cmd, term):
        if device not in self.devices:
            print "error: run command at nonexist device {0}".format(device)
            content = term + ',' + 'device nonexist'
            self.send_packet(pkt.DEVICE_CMD, content)
            return

        try:
            self.devices[device]['handle'].write(cmd+'\r')
            result='success'
            print "run command '{0}' at {1} succeed".format(cmd, device)
        except:
            if DEBUG: traceback.print_exc()
            result='fail'
            print "run command '{0}' at {1} failed".format(cmd, device)
        content = term + ',' + result
        self.send_packet(pkt.DEVICE_CMD, content)

    def login_and_get_server(self, controller_ip, controller_port):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        if ENCRYPT:
            cert_file = path.join(path.dirname(path.abspath(__file__)), 'controller_certificate.pem')
            sock = ssl.wrap_socket(sock, cert_reqs=ssl.CERT_REQUIRED, ca_certs = cert_file)
        try:
            sock.connect((controller_ip, controller_port))
            sock.settimeout(3)
            msg = ''
        except:
            if DEBUG: traceback.print_exc()
            return None

        content = pkt.construct(pkt.ACCESS_LOGIN, 'client,' + self.uuid)
        try:
            sock.send(content)
        except:
            if DEBUG: traceback.print_exc()
            sock.close()
            return None

        try:
            data = sock.recv(MAX_MSG_LENGTH)
        except KeyboardInterrupt:
            sock.close()
            raise
        except:
            sock.close()
            return None

        if data == '':
            sock.close()
            return None

        type, length, value, data = pkt.parse(data)
        #print 'controller', type, value
        rets = value.split(',')
        if type != pkt.ACCESS_LOGIN or rets[0] != 'success':
            accesskey_file = path.join(path.expanduser('~'), '.udclient', '.accesskey')
            if rets[0] == 'invalid access key' and path.exists(accesskey_file):
                os.remove(accesskey_file)
            print "login failed, ret={0}".format(value)
            sock.close()
            return None

        sock.close()
        return rets[1:]

    def connect_to_server(self, server_ip, server_port, certificate):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        if certificate != 'None':
            try:
                certfile = path.join(path.expanduser('~'), '.udclient', 'certificate.pem')
                f = open(certfile, 'wt')
                f.write(certificate)
                f.close()
                sock = ssl.wrap_socket(sock, cert_reqs=ssl.CERT_REQUIRED, ca_certs=certfile)
            except:
                if DEBUG: traceback.print_exc()
                return 'fail'
        try:
            sock.connect((server_ip, server_port))
            while self.output_queue.empty() == False:
                self.output_queue.get()
            self.service_socket = sock
            self.connected = True
            return "success"
        except:
            if DEBUG: traceback.print_exc()
            return "fail"

    def client_func(self, contoller_ip, controller_port):
        work_dir = path.join(path.expanduser('~'), '.udclient')
        if path.exists(work_dir) == False:
            os.mkdir(work_dir)
        signal.signal(signal.SIGINT, signal_handler)

        accesskey_file = path.join(path.expanduser('~'), '.udclient', '.accesskey')
        if path.exists(accesskey_file) == True:
            try:
                file = open(accesskey_file, 'rb')
                self.uuid = json.load(file)
                file.close()
            except:
                print "read access key failed"
        while self.uuid == None:
            try:
                uuid = raw_input("please input your access key: ")
            except:
                return
            is_valid_uuid = re.match('^[0-9a-f]{10,20}$', uuid)
            if is_valid_uuid == None:
                print "invalid access key {0}, please enter a valid access key".format(uuid)
                continue
            self.uuid = uuid
            try:
                file = open(accesskey_file, 'wb')
                json.dump(self.uuid, file)
                file.close()
            except:
                print "error: save access key to file failed"

        self.load_interfaces()

        thread.start_new_thread(self.packet_send_thread, ())
        thread.start_new_thread(self.debug_daemon_thread, ())
        thread.start_new_thread(self.device_monitor, ())

        file_received = {}
        file_receiving = {}
        self.connected = False
        self.service_socket = None
        msg = ''
        while True:
            try:
                if self.connected == False:
                    raise ConnectionLost

                new_msg = self.service_socket.recv(MAX_MSG_LENGTH)
                if new_msg == '':
                    raise ConnectionLost
                    break

                msg += new_msg
                while msg != '':
                    type, length, value, msg = pkt.parse(msg)
                    if type == pkt.TYPE_NONE:
                        break

                    for hash in list(file_receiving):
                        if time.time() > file_receiving[hash]['timeout']:
                            file_receiving[hash]['handle'].close()
                            try:
                                os.remove(file_receiving[hash]['name'])
                            except:
                                pass
                            file_receiving.pop(hash)

                    if type == pkt.FILE_BEGIN:
                        try:
                            [term, hash, fname] = value.split(':')
                        except:
                            print "argument error: {0} {1}".format(type, value)
                            continue
                        if hash in file_received:
                            if path.exists(file_received[hash]) == True:
                                content = term + ',' + 'exist'
                                self.send_packet(type, content)
                                continue
                            else:
                                file_received.pop(hash)

                        if hash in file_receiving:
                            content = term + ',' + 'busy'
                            self.send_packet(type, content)
                            continue

                        filename = path.join(path.expanduser('~'), '.udclient', path.basename(fname))
                        filename += '-' + term + '@' + time.strftime('%Y%m%d-%H%M%S')
                        filehandle = open(filename, 'wb')
                        timeout = time.time() + 5
                        file_receiving[hash] = {'name':filename, 'seq':0, 'handle':filehandle, 'timeout': timeout}
                        content = term + ',' + 'ok'
                        self.send_packet(type, content)
                        if DEBUG:
                            print 'start receiving {0} as {1}'.format(fname, filename)
                    elif type == pkt.FILE_DATA:
                        try:
                            split_value = value.split(':')
                            term = split_value[0]
                            hash = split_value[1]
                            seq  = split_value[2]
                            data = value[(len(term) + len(hash) + len(seq) + 3):]
                            seq = int(seq)
                        except:
                            print "argument error: {0}".format(type)
                            continue
                        if hash not in file_receiving:
                            content = term + ',' + 'noexist'
                            self.send_packet(type, content)
                            continue
                        if file_receiving[hash]['seq'] != seq and file_receiving[hash]['seq'] != seq + 1:
                            content = term + ',' + 'seqerror'
                            self.send_packet(type, content)
                            continue
                        if file_receiving[hash]['seq'] == seq:
                            file_receiving[hash]['handle'].write(data)
                            file_receiving[hash]['seq'] += 1
                            file_receiving[hash]['timeout'] = time.time() + 5
                        content = term + ',' + 'ok'
                        self.send_packet(type, content)
                    elif type == pkt.FILE_END:
                        try:
                            [term, hash, fname] = value.split(':')
                        except:
                            print "argument error: {0} {1}".format(type, value)
                            continue
                        if hash not in file_receiving:
                            content = term + ',' + 'noexist'
                            self.send_packet(type, content)
                            continue
                        file_receiving[hash]['handle'].close()
                        localhash = pkt.hash_of_file(file_receiving[hash]['name'])
                        if localhash != hash:
                            response = 'hasherror'
                        else:
                            response = 'ok'
                            file_received[hash] = file_receiving[hash]['name']
                        if DEBUG:
                            print 'finished receiving {0}, result:{1}'.format(file_receiving[hash]['name'], response)
                        file_receiving.pop(hash)
                        content = term + ',' + response
                        self.send_packet(type, content)
                    elif type == pkt.DEVICE_ERASE:
                        try:
                            [term, device] = value.split(',')
                        except:
                            print "argument error: {0} {1}".format(type, value)
                            continue
                        if device in self.devices:
                            if self.devices[device]['ucmd_queue'].full() == False:
                                self.devices[device]['ucmd_queue'].put([type, term])
                                continue
                            else:
                                result = 'busy'
                                print 'erase', device,  'failed, device busy'
                        else:
                            result = 'nonexist'
                            print 'erase', device,  'failed, device nonexist'
                        content = term + ',' + result
                        self.send_packet(type, content)
                    elif type == pkt.DEVICE_PROGRAM:
                        try:
                            [term, device, address, hash] = value.split(',')
                        except:
                            print "argument error: {0} {1}".format(type, value)
                            continue
                        if hash not in file_received:
                            content = term + ',' + 'error'
                            self.send_packet(type, content)
                            continue
                        filename = file_received[hash]
                        if device in self.devices:
                            if self.devices[device]['ucmd_queue'].full() == False:
                                self.devices[device]['ucmd_queue'].put([type, term, address, filename])
                                continue
                            else:
                                result = 'busy'
                                print 'program {0} to {1} @ {2} failed, device busy'.format(filename, device, address)
                        else:
                            result = 'error'
                            print 'program {0} to {1} @ {2} failed, device nonexist'.format(filename, device, address)
                        content = term + ',' + result
                        self.send_packet(type, content)
                    elif type in [pkt.DEVICE_RESET, pkt.DEVICE_START, pkt.DEVICE_STOP]:
                        operations = {pkt.DEVICE_RESET:'reset', pkt.DEVICE_START:'start', pkt.DEVICE_STOP:'stop'}
                        try:
                            [term, device] = value.split(',')
                        except:
                            print "argument error: {0} {1}".format(type, value)
                            continue
                        if device in self.devices and self.devices[device]['valid'] == True:
                            if self.devices[device]['ucmd_queue'].full() == False:
                                self.devices[device]['ucmd_queue'].put([type, term])
                                continue
                            else:
                                result = 'busy'
                                print operations[type], device, 'failed, device busy'
                        else:
                            result = 'nonexist'
                            print operations[type], device, 'failed, device nonexist'
                        content = term + ',' + result
                        self.send_packet(type, content)
                    elif type in [pkt.DEVICE_DEBUG_START, pkt.DEVICE_DEBUG_REINIT, pkt.DEVICE_DEBUG_STOP]:
                        operations = {
                            pkt.DEVICE_DEBUG_START:'start debuging',
                            pkt.DEVICE_DEBUG_START:'reinit debuging',
                            pkt.DEVICE_STOP:'stop debugging'}
                        try:
                            [term, device] = value.split(',')
                        except:
                            print "argument error: {0} {1}".format(type, value)
                            continue
                        debug_cmd = [type, term, device]
                        try:
                            self.debug_queue.put_nowait(debug_cmd)
                        except:
                            print "error: {0} debug session failed, debug_queue full".format(operations[type])
                            content = term + ',' + device + ':' + 'daemon busy'
                            self.send_packet(type, content)
                    elif type == pkt.DEVICE_CMD:
                        term_dev = value.split(':')[0]
                        term_dev_len = len(term_dev) + 1
                        try:
                            [term, device] = term_dev.split(',')
                        except:
                            print "argument error: {0} {1}".format(type, value)
                            continue
                        cmd = value[term_dev_len:]
                        if type == pkt.DEVICE_CMD:
                            cmd = cmd.replace('|', ' ')
                        if device in self.devices and self.devices[device]['valid'] == True:
                            if self.devices[device]['ucmd_queue'].full() == False:
                                self.devices[device]['ucmd_queue'].put([type, term, cmd])
                                continue
                            else:
                                result = 'busy'
                                print "run command '{0}' at {1} failed, device busy".format(cmd, device)
                        else:
                            result = 'nonexist'
                            print "run command '{0}' at {1} failed, device nonexist".format(cmd, device)
                        content = term + ',' + result
                        self.send_packet(type, content)
                    elif type == pkt.DEVICE_DEBUG_DATA:
                        term_dev = value.split(':')[0]
                        term_dev_len = len(term_dev) + 1
                        try:
                            [term, device] = term_dev.split(',')
                        except:
                            print "argument error: {0} {1}".format(type, value)
                            continue

                        data = value[term_dev_len:]
                        if device not in self.devices or 'debug_socket' not in self.devices[device]:
                            content = term + ',' + device + ':' + 'session nonexist'
                            self.send_packet(pkt.DEVICE_DEBUG_STOP, content)
                            return

                        try:
                            self.devices[device]['debug_socket'].send(data)
                        except:
                            if DEBUG: traceback.print_exc()
                            print "forward debug data to {1} failed".format(device)
                    elif type == pkt.CLIENT_LOGIN:
                        if value == 'request':
                            print 'server request login'
                            data = self.uuid + ',' + self.poll_str + ',' + token
                            self.send_packet(pkt.CLIENT_LOGIN, data)
                        elif value == 'success':
                            print "login to server succeed"
                            self.send_device_list()
                            self.send_device_status()
                        else:
                            print "login to server failed, retry later ..."
                            try:
                                time.sleep(10)
                            except KeyboardInterrupt:
                                break
                            raise ConnectionLost
            except ConnectionLost:
                self.connected = False
                if self.service_socket != None:
                    self.service_socket.close()
                    print 'connection to server lost, try reconnecting...'

                try:
                    result = self.login_and_get_server(contoller_ip, controller_port)
                except KeyboardInterrupt:
                    break
                if result == None:
                    print 'login to controller failed, retry later...'
                    try:
                        time.sleep(5)
                    except KeyboardInterrupt:
                        break
                    continue

                try:
                    [server_ip, server_port, token, certificate] = result
                    server_port = int(server_port)
                    print 'login to controller succees, server_ip-{0} server_port-{1}'.format(server_ip, server_port)
                except:
                    print 'login to controller failed, invalid return={0}'.format(result)
                    try:
                        time.sleep(5)
                    except KeyboardInterrupt:
                        break
                    continue

                result = self.connect_to_server(server_ip, server_port, certificate)
                if result == 'success':
                    print 'connect to server {0}:{1} succeeded'.format(server_ip, server_port)
                else:
                    print 'connect to server {0}:{1} failed, retry later ...'.format(server_ip, server_port)
                    try:
                        time.sleep(5)
                    except KeyboardInterrupt:
                        break
                    continue

                data = self.uuid + ',' + self.poll_str + ',' + token
                self.send_packet(pkt.CLIENT_LOGIN, data)
            except KeyboardInterrupt:
                break
            except:
                if DEBUG: traceback.print_exc()
        print "client exiting ..."
        self.keep_running = False
        time.sleep(0.3)
        if self.service_socket: self.service_socket.close()



import os, sys
import click
from aos.util import log, error, popen, get_config_value
from aos.config import *
from aos.commands.upload import _call_scons_target

# Make command
@click.command("debug", short_help="Start gdb server")
@click.argument("target", required=False, nargs=-1)
@click.option("--work-path", "-w", help="Alternative work path if aos_sdk_path unavailable")
@click.option("--binaries-dir", "-b", help="Directory to store debug binaries")
@click.option("--start-client", "-c", is_flag=True, help="Start gdb client")
@click.option("--gdb-args", "-g", help="Arguments pass to gdb client")
def cli(target, work_path, binaries_dir, start_client, gdb_args):
    """ Initialize debug env for aos image. """
    from aos import __version__
    log('aos-cube version: %s\n' % __version__)

    debug_target = ' '.join(target)

    if work_path:
        if os.path.isdir(work_path):
            aos_path = work_path
        else:
            error("Can't find dir %s" % work_path)
    else:
        #cd to aos root_dir
        ret, original_dir = cd_aos_root()
        if ret != 'success':
            log("[INFO]: Not in AliOS-Things source code directory")
            log("[INFO]: Current directory is: '%s'\n" % original_dir)
            if os.path.isdir(original_dir):
                os.chdir(original_dir)

            aos_path = Global().get_cfg(AOS_SDK_PATH)
            if aos_path == None:
                error("[ERRO]: Not in aos_sdk_path, aos_sdk unavailable as well!")
            else:
                log("[INFO]: Config Loading OK, use '%s' as sdk path\n" % aos_path)
        else:
            aos_path = os.getcwd()
            log("[INFO]: Currently in aos_sdk_path: '%s'\n" % os.getcwd())

    # read app & board from .config
    if debug_target == '':
        # check AliOS Things version
        if os.path.exists(os.path.join(aos_path, 'build', 'Config.in')) == False:
            error('Target invalid')

        board = None
        app = None
        log("[INFO]: Not set target, read target from .config\n")

        config_file = os.path.join(aos_path, '.config')
        if os.path.exists(config_file) == False:
            error('Config file NOT EXIST: %s\n' % config_file)

        board = get_config_value('AOS_BUILD_BOARD', config_file)
        app = get_config_value('AOS_BUILD_APP', config_file)

        if not app and not board:
            error('None target in %s\n' % config_file)
        debug_target = '%s@%s' % (app, board)

    elif '@' not in debug_target or len(debug_target.split('@')) != 2:
        error('Target invalid')

    if work_path and not os.path.isdir(work_path):
        error("Can't find dir %s" % work_path)

    if binaries_dir and not os.path.isdir(binaries_dir):
        error("Can't find dir %s" % binaries_dir)

    extra_args = []
    if work_path and os.path.isdir(work_path):
        extra_args += ['WORKPATH=' + work_path]

    if binaries_dir and os.path.isdir(binaries_dir):
        extra_args += ['BINDIR=' + binaries_dir]

    if start_client:
        extra_args += ['STARTCLIENT=True']

    if gdb_args:
        extra_args += ['GDBARGS="' + gdb_args + '"']

    click.secho("[INFO]: Target: %s\n" % debug_target, fg="green")
    _call_scons_target(debug_target, 'debug', aos_path, extra_args)
